function [data] = combineplaner(cfg,data)

% COMBINEPLANAR computes the planar gradient magnitude over both directions
% combining the two gradients at each sensor to a single positive-valued number.
% This can be done for averaged ERFs or TFRs (i.e. powerspectra). 
% 
% Use as
%   [data] = combineplanar(cfg, data)
% where data contains an averaged planar gradient (either ERF or TFR).
%
% In the case of ERFs, the configuration can contain
%   cfg.blc       = 'yes' or 'no' (default)
%   cfg.blcwindow = [begin end]
%
% After combining the planar data, the planar gradiometer definition does not 
% match the data any more and therefore it is removed from the data. With 
%   cfg.combinegrad  = 'yes' 
% the function will try to reconstruct the axial gradiometer definition.
%
% See also MEGINTERPOLATE, MEGPLANAR

% Copyright (C) 2004, Ole Jensen, Robert Oostenveld
%
% $Log: combineplanar.m,v $
% Revision 1.16  2005/02/18 12:57:50  roboos
% added support for singletrial power spectra
%
% Revision 1.15  2004/11/12 09:21:01  roboos
% added detection for 306 channel Neuromag data, no actual implementation yet
%
% Revision 1.14  2004/06/23 20:28:54  roberto
% added support for 122 channels, indicating Neuromag data
%
% Revision 1.13  2004/06/03 15:46:27  roberto
% added warning message for planar gradiometers
%
% Revision 1.12  2004/04/28 09:33:44  roberto
% added check for missing gradiometer definition
%
% Revision 1.11  2004/04/22 08:48:17  roberto
% added cfg.combinegrad, wrapped around planar to axial conversion
%
% Revision 1.10  2004/04/20 20:09:29  roberto
% convert input (planar) gradiometer definition into axial gradiometer definition,
% some changes in splaces and tabs
%
% Revision 1.9  2004/04/13 16:31:09  roberto
% fixed bug in dbstack selection of function filename for Matlab 6.1
%
% Revision 1.8  2004/04/13 14:25:24  roberto
% wrapped code-snippet around mfilename to make it compatible with Matlab 6.1
%
% Revision 1.7  2004/04/06 20:00:40  roberto
% minor changes in documentation
%
% Revision 1.6  2004/03/29 15:13:29  roberto
% added version and history to output configuration
%
% Revision 1.5  2004/03/23 11:01:09  roberto
% implemented support to keep non-MEG channels in the data after combining the planar gradients
%
% Revision 1.4  2004/02/10 15:45:04  roberto
% fixed bug in cfg.blcwindow and changed its default into 'all'
%
% Revision 1.3  2004/01/27 17:02:49  roberto
% added support for frequency and time-frequency data (only singletrial)
% removed custom baselinecorrection and replaced with blc function
%
% Revision 1.2  2004/01/27 09:49:26  roberto
% fixed small bug in length(data.label)~=302
%
% Revision 1.1  2004/01/27 09:42:06  roberto
% took over this function from Ole, improved compatibility with framework,
% added help and built in some checks for invalid input data
%

% set the defaults
if ~isfield(cfg, 'blc'),         cfg.blc         = 'no';  end
if ~isfield(cfg, 'blcwindow'),   cfg.blcwindow   = 'all'; end
if ~isfield(cfg, 'combinegrad'), cfg.combinegrad = 'no';  end
if isfield(cfg, 'baseline')
  warning('only supporting cfg.baseline for backwards compatibility, please update your cfg');
  cfg.blc = 'yes';
  cfg.blcwindow = cfg.baseline;
end
if strcmp(cfg.blc, 'yes') & isempty(cfg.blcwindow)
  cfg.blcwindow = [min(data.time) max(data.time)];
end

% check the input data
if ~isfield(data, 'avg') & ~isfield(data, 'powspctrm')
  error('unsupported data format')
end

% Find the two mathing channels for each planar gradiometer.
% This assumes that the channels are labeled XX_dH and xx_dV, which is 
% only the case if they were computed with megplanar for our 151 channel CTF system 
sel_dH = []; sel_dV = []; sel_other = [];
lab_dH = {}; lab_dV = {}; lab_other = {};
for i=1:length(data.label)
  if findstr(data.label{i}, '_dH')
    sel_dH(end+1) = i;
    lab_dH{end+1} = data.label{i}(1:(end-3));	% remove the "_dH" part
  elseif findstr(data.label{i}, '_dV')
    sel_dV(end+1) = i;
    lab_dV{end+1} = data.label{i}(1:(end-3));	% remove the "_dV" part
  else
    sel_other(end+1) = i;
    lab_other{end+1} = data.label{i};
  end
end
if length(sel_dH)~=length(sel_dV)
  error('could not find all matching planar channels')
end
[ind_dH, ind_dV] = match_str(lab_dH, lab_dV);
sel_dH = sel_dH(ind_dH);
sel_dV = sel_dV(ind_dV);

% if no matching channels were found, it could be because it is 122 channel
% Neuromag data (which also is planar gradients).
if isempty(sel_dH) & isempty(sel_dV)
  if length(data.label)==122
    fprintf('looks as if it is 122 channel Neuromag planar gradient data\n');
    % channel 001 and 002 belong to each other, etc.
    sel_dH = 1:2:121;
    sel_dV = 2:2:122;
    sel_other = [];
    lab_dH = data.label(sel_dH);
    lab_dV = data.label(sel_dV);
    lab_other = {};
  elseif length(data.label)==306
    fprintf('looks as if it is 306 channel Neuromag data\n');
    % should channel 001 and 002 be combined, skipping channel 003?
    % this needs to be checked before it gets implemented here
    error('unsupported data type');
  end
end      

% perform baseline correction
if strcmp(cfg.blc, 'yes') 
  if ~isfield(data, 'avg')
    error('baseline correction is only supported for ERFs')
  else
    if ischar(cfg.blcwindow) & strcmp(cfg.blcwindow, 'all')
      cfg.blcwindow = [min(data.time) max(data.time)];
    end
    % find the timebins corresponding to the baseline interval
    tbeg = nearest(data.time, cfg.blcwindow(1));
    tend = nearest(data.time, cfg.blcwindow(2));
    cfg.blcwindow(1) = data.time(tbeg);
    cfg.blcwindow(2) = data.time(tend);
    data.avg = blc(data.avg, tbeg, tend); 
  end
end

if isfield(data, 'powspctrm')
  % compute the power of each planar channel, simply by adding the power
  if strcmp(data.dimord, 'sgncmb_frq')
    tmp1 = data.powspctrm(sel_dH,:) + data.powspctrm(sel_dV,:);
    tmp2 = data.powspctrm(sel_other,:);
    data.powspctrm = [tmp1; tmp2];
  elseif strcmp(data.dimord, 'sgncmb_frq_tim')
    tmp1 = data.powspctrm(sel_dH,:,:) + data.powspctrm(sel_dV,:,:);
    tmp2 = data.powspctrm(sel_other,:,:);
    data.powspctrm = cat(1, tmp1, tmp2);
  elseif strcmp(data.dimord, 'rpt_sgncmb_frq')
    tmp1 = data.powspctrm(:,sel_dH,:) + data.powspctrm(:,sel_dV,:);
    tmp2 = data.powspctrm(:,sel_other,:);
    data.powspctrm = cat(2, tmp1, tmp2);
  elseif strcmp(data.dimord, 'rpt_sgncmb_frq_tim')
    tmp1 = data.powspctrm(:,sel_dH,:,:) + data.powspctrm(:,sel_dV,:,:);
    tmp2 = data.powspctrm(:,sel_other,:,:);
    data.powspctrm = cat(2, tmp1, tmp2);
  else
    error('unsupported dimension order of frequency data');
  end    
elseif isfield(data, 'avg')
  % compute the magnitude of each planar channel, simply by pythagoras
  tmp1 = sqrt(data.avg(sel_dH,:).^2 + data.avg(sel_dV,:).^2);
  tmp2 = data.avg(sel_other,:);
  data.avg = [tmp1; tmp2];
end

if strcmp(cfg.combinegrad, 'no') & ~isfield(data, 'grad')
  % the planar gradiometer definition was already removed
  % nothing needs to be done here
elseif strcmp(cfg.combinegrad, 'no') & isfield(data, 'grad')
  % remove the planar gradiometer definition since it does not match the data any more
  data = rmfield(data, 'grad');
elseif strcmp(cfg.combinegrad, 'yes') & ~isfield(data, 'grad')
  % there is no gradiometer definition, impossible to reconstruct it
  error('the planar gradiometer definition is missing, cannot convert it back to axial');
elseif strcmp(cfg.combinegrad, 'yes') & isfield(data, 'grad')
  warning('trying to convert planar to axial gradiometers, this is experimental');
  % try to reconstruct the original axial gradiometer array from the planar gradiometer definition
  orig = data.grad;
  if all(size(orig.pnt)==[302 3]) & ...
    all(size(orig.pnt)==[302 3]) & ...
    all(size(orig.tra)==[302 302]) & ...
    length(orig.label)==302 & ...
    all(sum(orig.tra~=0,1)>2)
    % This looks as if it was made using the MEGPLANAR nearest neighbour approach
    % which means that the coil position and orientation still correspond
    % with those of the original axial gradiometer. Only the label and tra
    % have been modified and have to be restored to their original values.
    axial.pnt = orig.pnt;
    axial.ori = orig.ori;
    for i=1:151
      axial.label{i} = orig.label{i}(1:(end-3));
    end
    if all(orig.ori(1,:)==orig.ori(152,:))
      % orientation is the same, the subtraction should be in "tra"
      axial.tra = [eye(151) -eye(151)];
    else
      % orientation is opposite, the subtraction should not be in "tra"
      axial.tra = [eye(151) eye(151)];
    end
    try
      axial.unit = orig.unit;
    end
  else
    error('cannot convert gradiometer definition back to axial, please contact Robert');
  end
  data.grad = axial;
end

% reconstruct the original channel labels
data.label = [lab_dH(:); lab_other(:)];

% remove all fields for which the planar gradient could not be combined
try, rmfield(data, 'crsspctrm');   end
try, rmfield(data, 'sgncmb');      end
try, rmfield(data, 'var');         end
try, rmfield(data, 'cov');         end

% store the configuration of this function call, including that of the previous function call
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id  = '$Id: combineplanar.m,v 1.16 2005/02/18 12:57:50 roboos Exp $';
cfg.previous    = data.cfg;
data.cfg        = cfg;    % remember the full configuration details
