function [lf] = compute_leadfield(pos, sens, vol);

% COMPUTE_LEADFIELD computes a forward solution for a dipole in a a volume
% conductor model. The forward solution is expressed as the leadfield
% matrix (Nchan*3), where each column corresponds with the potential or field
% distributions on all sensors for one of the x,y,z-orientations of the
% dipole.
%
% Use as
%   [lf] = compute_leadfield(pos, sens, vol)
% with input arguments
%   pos    position dipole (1x3 or Nx3)
%   sens   structure with gradiometer or electrode definition
%   vol    structure with volume conductor definition
%
% Depending on the specific input arguments for the sensor and volume, this
% function will select the appropriate low-level EEG or MEG forward model.
% The supported forward solutions are
%   EEG, single sphere
%   MEG, single sphere
%   EEG, concentric spheres (max 4)
%   MEG, multiple spheres (one sphere per channel)
%   EEG, using leadfield interpolation on precomputed grid
%   MEG, using leadfield interpolation on precomputed grid
%   EEG, BEM (using ASA to precompute the sytem matrix)
%
% The leadfield matrix for EEG will have an average reference over all
% the electrodes.

% Supported in the hopefully near future
%   EEG+MEG, leadfield interpolation
%   EEG+MEG, BEM (using Neuromag software)
%   MEG, Guido Nolte's algorithm

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: compute_leadfield.m,v $
% Revision 1.5  2005/02/21 08:01:19  roboos
% removed spaces at the end of line, no code changes
%
% Revision 1.4  2005/02/08 11:59:14  roboos
% added an extra check on the input
%
% Revision 1.3  2004/10/25 16:20:13  roboos
% fixed bug in selection of electrode positions from sens structure
%
% Revision 1.2  2004/09/21 15:31:51  roboos
% renamed grad into sens
%
% Revision 1.1  2004/08/19 08:18:20  roboos
% new implementations, generalized for EEG and MEG
%

% determine whether it is EEG or MEG
if ~isstruct(sens) & size(sens,2)==3
  % definition of electrode positions only, restructure it
  sens.pnt = sens;
  iseeg = 1;
  ismeg = 0;
elseif isfield(sens, 'type') & strcmp(sens.type, 'electrodes')
  iseeg = 1;
  ismeg = 0;
elseif isfield(sens, 'type') & strcmp(sens.type, 'gradiometers')
  iseeg = 0;
  ismeg = 1;
else
  iseeg = 0;
  ismeg = 0;
end

if ismeg & iseeg
  % this is something that I might implement in the future
  error('simultaneous EEG and MEG not supported');

elseif ~ismeg & ~iseeg
  error('the forward model does not look like EEG, nor like MEG');

elseif ismeg
  if isfield(vol, 'r') & all(size(vol.r)==[1 1])
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % MEG single-sphere volume conductor model
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if isfield(sens, 'pnt1') & isfield(sens, 'pnt2')
      % this appears to be an old fashioned gradiometer definition
      % add the magnetometer positions at the upper coil
      nchan = size(sens.pnt1, 1);
      pnt = [sens.pnt1; sens.pnt2];
      ori = [sens.ori1; sens.ori2];
    else
      pnt = sens.pnt;
      ori = sens.ori;
    end

    if isfield(vol, 'o')
      % shift dipole and magnetometers to origin of sphere
      pos = pos - repmat(vol.o, size(pos,1), 1);
      pnt = pnt - repmat(vol.o, size(pnt,1), 1);
    end

    if size(pos,1)>1
      % loop over multiple dipoles
      lf = zeros(size(pnt,1),3*size(pos,1));
      for i=1:size(pos,1)
        lf(:,(3*i-2):(3*i)) = meg_leadfield1(pos(i,:), pnt, ori);
      end
    else
      % only single dipole
      lf = meg_leadfield1(pos, pnt, ori);
    end

    if isfield(sens, 'pnt2')
      % this appears to be a gradiometer definition
      % construct the gradient from both magnetometers
      lf = lf(1:nchan, :) - lf((nchan+1):end,:);
    end

    if isfield(sens, 'tra')
      % this appears to be the modern complex gradiometer definition
      % construct the channels from a linear combination of all magnetometers
      lf = sens.tra * lf;
    end

  elseif isfield(vol, 'r') & ~all(size(vol.r)==[1 1])
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % MEG multi-sphere volume conductor model
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    nspheres = length(vol.r);
    if size(vol.o, 1) ~= nspheres
      error('different number of spheres for the radius and origin')
    end

    if isfield(sens, 'pnt1') & isfield(sens, 'pnt2')
      error('oldfashoned gradiometer format not supported for multi-sphere headmodel');
    end

    if size(sens.pnt)~=nspheres
      error('number of spheres is not equal to the number of coils');
    end

    ndipoles = size(pos,1);
    lf = zeros(nspheres, 3*ndipoles);
    for chan=1:nspheres
      for dip=1:ndipoles
        % shift dipole and magnetometer coil to origin of sphere
        dippos = pos(dip,:)       - vol.o(chan,:);
        chnpos = sens.pnt(chan,:) - vol.o(chan,:);
        tmp = meg_leadfield1(dippos, chnpos, sens.ori(chan,:));
        lf(chan,(3*dip-2):(3*dip)) = tmp;
      end
    end

    if isfield(sens, 'tra')
      % this appears to be the modern complex gradiometer definition
      % construct the channels from a linear combination of all magnetometers
      lf = sens.tra * lf;
    end

  elseif isempty(vol)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % magnetic dipole instead of electric (current) dipole in an infinite vacuum
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    persistent warning_issued;
    if isempty(warning_issued)
      % give the warning only once
      warning('assuming magnetic dipole in infinite vacuum');
      warning_issued = 1;
    end

    if isfield(sens, 'pnt1') & isfield(sens, 'pnt2')
      % this appears to be an old fashioned gradiometer definition
      % add the magnetometer positions at the upper coil
      nchan = size(sens.pnt1, 1);
      pnt = [sens.pnt1; sens.pnt2];
      ori = [sens.ori1; sens.ori2];
    else
      pnt = sens.pnt;
      ori = sens.ori;
    end

    if size(pos,1)>1
      % loop over multiple dipoles
      lf = zeros(size(pnt,1),3*size(pos,1));
      for i=1:size(pos,1)
        lf(:,(3*i-2):(3*i)) = magnetic_dipole(pos(i,:), pnt, ori);
      end
    else
      % only single dipole
      lf = magnetic_dipole(pos, pnt, ori);
    end

    if isfield(sens, 'pnt2')
      % this appears to be a gradiometer definition
      % construct the gradient from both magnetometers
      lf = lf(1:nchan, :) - lf((nchan+1):end,:);
    end

    if isfield(sens, 'tra')
      % this appears to be a complex gradiometer definition
      % construct the channels from a linear combination of all magnetometers
      lf = sens.tra * lf;
    end

  else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % MEG unrecognized volume conductor model
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    error('unrecognized volume conductor model for MEG');
  end

elseif iseeg
  if isfield(vol, 'r')
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % EEG spherical volume conductor model
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % FIXME, this is not consistent between spherical and BEM
    % sort the spheres from the smallest to the largest
    [vol.r, indx] = sort(vol.r);
    vol.c = vol.c(indx);

    Nspheres = length(vol.c);
    if length(vol.r)~=Nspheres
      error('the number of spheres in the volume conductor model is ambiguous');
    end

    if isfield(vol, 'o')
      % shift the origin of the spheres, electrodes and dipole
      sens.pnt = sens.pnt - repmat(vol.o, size(sens.pnt,1), 1);
      pos = pos - repmat(vol.o, size(pos,1), 1);
    end

    if Nspheres==1
      if size(pos,1)>1
        % loop over multiple dipoles
        lf = zeros(size(sens.pnt,1),3*size(pos,1));
        for i=1:size(pos,1)
          lf(:,(3*i-2):(3*i)) = eeg_leadfield1(pos(i,:), sens.pnt, vol);
        end
      else
        % only single dipole
        lf = eeg_leadfield1(pos, sens.pnt, vol);
      end

    elseif Nspheres==2
      vol.r = [vol.r(1) vol.r(2) vol.r(2) vol.r(2)];
      vol.c = [vol.c(1) vol.c(2) vol.c(2) vol.c(2)];
      if size(pos,1)>1
        % loop over multiple dipoles
        lf = zeros(size(sens.pnt,1),3*size(pos,1));
        for i=1:size(pos,1)
          lf(:,(3*i-2):(3*i)) = eeg_leadfield4(pos(i,:), sens.pnt, vol);
        end
      else
        % only single dipole
        lf = eeg_leadfield4(pos, sens.pnt, vol);
      end

    elseif Nspheres==3
      vol.r = [vol.r(1) vol.r(2) vol.r(3) vol.r(3)];
      vol.c = [vol.c(1) vol.c(2) vol.c(3) vol.c(3)];
      if size(pos,1)>1
        % loop over multiple dipoles
        lf = zeros(size(sens.pnt,1),3*size(pos,1));
        for i=1:size(pos,1)
          lf(:,(3*i-2):(3*i)) = eeg_leadfield4(pos(i,:), sens.pnt, vol);
        end
      else
        % only single dipole
        lf = eeg_leadfield4(pos, sens.pnt, vol);
      end

    elseif Nspheres==4
      if size(pos,1)>1
        % loop over multiple dipoles
        lf = zeros(size(sens.pnt,1),3*size(pos,1));
        for i=1:size(pos,1)
          lf(:,(3*i-2):(3*i)) = eeg_leadfield4(pos(i,:), sens.pnt, vol);
        end
      else
        % only single dipole
        lf = eeg_leadfield4(pos, sens.pnt, vol);
      end

    else
      error('more than 4 concentric spheres are not supported');
    end

  elseif isfield(vol, 'bnd')
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % EEG boundary element method volume conductor model
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    lf = eeg_leadfieldb(pos, sens.pnt, vol);

  else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % MEG unrecognized volume conductor model
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    error('unrecognized volume conductor model for EEG');
  end

  % compute average reference for EEG leadfield
  avg = mean(lf, 1);
  lf  = lf - repmat(avg, size(lf,1), 1);

end % elseif iseeg
