function [source] = dipolefitting(cfg, data)

% DIPOLEFITTING scanning and non-linear fit with small number of dipoles
%
% This function will initially scan the whole brain with a single
% dipole on a regular coarse grid, and subsequently start at the most
% optimal location with a non-linear search. Alternatively you can
% specify the initial location of the dipole(s) and the non-linear
% search will start from there.
% 
% Multiple components of an ICA/PCA can be fitted simultaneously, in which
% case one dipole model will be fitted to all components combined.
% You can compare this to a spatio-temporal dipole model. The dipole model
% will try to explain all components.
% 
% Use as
%   [source] = dipolefitting(cfg, timelock)
%   [source] = dipolefitting(cfg, grandavg)
%   [source] = dipolefitting(cfg, comp)
%
% The data that can be fitted is the output from the TIMELOCKANALYSIS,
% TIMELOCKGRANDAVERAGE and the COMPONENTANALYSIS functions.
% 
% The configuration may contain the following fields
%   cfg.dip.pos     = initial dipole position, matrix of Ndipoles x 3
%   cfg.gridsearch  = 'yes' or 'no'
%   cfg.nonlinear   = 'yes' or 'no'
%   cfg.model       = 'moving' | 'regional'
%   cfg.grid        = predefined grid for scanning
%   cfg.xgrid       = -12:1:12 for initial scanning, can be 'auto'
%   cfg.ygrid       = -12:1:12 for initial scanning, can be 'auto'
%   cfg.zgrid       =   0:1:13 for initial scanning, can be 'auto'
%   cfg.resolution  = number, for automatic grid generation
%   cfg.latency     = [begin end] or 'all'
%   cfg.component   = array with numbers (can be empty -> all)
%   cfg.symmetry    = 'x' or 'y' symmetry for two dipoles (can be empty)
%   cfg.numdipoles  = number, default is 1
%   cfg.channel     = cell-array (default is 'all', see CHANNELSELECTION)
%   cfg.hdmfile     = filename of headmodel, or
%   cfg.vol         = headmodel
%   cfg.gradfile    = filename of gradiometer definition, or
%   cfg.grad        = gradiometer definition
%   cfg.elecfile    = filename of electrodes, or
%   cfg.elec        = electrodes
%
% A moving dipole model has a different position (and orientation) for each timepoint, 
% or for each component. A regional dipole model has the same position for each
% timepoint or component, and a different orientation.
%
% See also SOURCEANALYSIS

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: dipolefitting.m,v $
% Revision 1.14  2005/02/21 07:56:54  roboos
% deleted the subfunction compute_leadfield, it is now a separate function
%
% Revision 1.13  2004/12/08 18:00:14  roboos
% implemented consistent method of selecting a subset of channels for
% forward and inverse computations using cfg.channel and updated the
% ducumentation
%
% Revision 1.12  2004/09/06 07:47:03  roboos
% fixed bug for finding optimium after grid scanning fixed dipole model
%
% Revision 1.11  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%
% Revision 1.10  2004/08/30 13:18:06  roboos
% fixed 2 bogs for moving dipole model
%
% Revision 1.9  2004/06/21 20:13:25  roberto
% fixed two bugs: one with sens.pnt and the other with inside/outside dipole positions after scanning
%
% Revision 1.8  2004/06/03 15:47:21  roberto
% improved detecton of correct configuration (numdipoles)
%
% Revision 1.7  2004/05/19 15:39:35  roberto
% fixed bug in time, which can be cell-array in case of ICA on raw data
% added complete version details to output configuration
%
% Revision 1.6  2004/05/13 23:35:47  roberto
% fixed multiple bugs related to new implementation moving/regional
%
% Revision 1.5  2004/05/13 22:36:30  roberto
% implemented cfg.method = 'regional' (old behaviour) and 'moving' (additional)
%
% Revision 1.4  2004/03/06 13:04:03  roberto
% fixed some small bugs and added time/component number to the output
%
% Revision 1.3  2004/02/05 12:20:06  roberto
% moved the code that was common with dipolefitting into separate prepare_grid and prepare_sens_vol functions
% fixed multiple small bugs
%
% Revision 1.2  2004/01/27 14:39:05  roberto
% multiple small bug fixes
%
% Revision 1.1  2004/01/22 21:42:52  roberto
% initial version, not tested
% support for single dipole scanning, fitting and symmetric dipole pair
%

% set the defaults
if ~isfield(cfg, 'channel'),     cfg.channel = 'all';        end
if ~isfield(cfg, 'latency'),     cfg.latency = 'all';        end
if ~isfield(cfg, 'component'),   cfg.component = [];         end
if ~isfield(cfg, 'symmetry'),    cfg.symmetry = [];          end
if ~isfield(cfg, 'gridsearch'),  cfg.gridsearch = 'yes';     end
if ~isfield(cfg, 'nonlinear'),   cfg.nonlinear = 'yes';      end
if ~isfield(cfg, 'resolution'),  cfg.resolution = 1;         end
if ~isfield(cfg, 'xgrid'),       cfg.xgrid = 'auto';         end
if ~isfield(cfg, 'ygrid'),       cfg.ygrid = 'auto';         end
if ~isfield(cfg, 'zgrid'),       cfg.zgrid = 'auto';         end

% the default for this depends on the data type
if ~isfield(cfg, 'model'),
  if ~isempty(cfg.component)
    % each component is fitted independently
    cfg.model = 'moving';
  elseif ~isempty(cfg.latency)
    % all latencies are fitted together
    cfg.model = 'regional';
  end
end

if ~isfield(cfg, 'numdipoles')
  if isfield(cfg, 'dip')
    cfg.numdipoles = size(cfg.dip(1).pos,1);
  else
    cfg.numdipoles = 1;
  end
end

% set up the symmetry constraints
if ~isempty(cfg.symmetry)
  if cfg.numdipoles~=2
    error('symmetry constraints are only supported for two-dipole models');
  elseif strcmp(cfg.symmetry, 'x')
    cfg.constr.reduce = [1 2 3];
    cfg.constr.expand = [1 2 3 1 2 3];
    cfg.constr.mirror = [1 1 1 -1 1 1];
  elseif strcmp(cfg.symmetry, 'y')
    cfg.constr.reduce = [1 2 3];
    cfg.constr.expand = [1 2 3 1 2 3];
    cfg.constr.mirror = [1 1 1 1 -1 1];
  else
    error('unrecognized symmetry constraint');
  end
else
  % no constraints have been specified
  cfg.constr = [];
end

if isfield(data, 'topolabel')
  % rename the labels for convenience
  data.timecourselabel = data.label;
  data.label = data.topolabel;
end

% prepare the volume conduction model and the sensor array
[vol, sens, cfg] = prepare_vol_sens(cfg, data);

% select the channels in the data
[seldata, selsens] = match_str(data.label, sens.label);

nchans = length(seldata);
fprintf('using %d channels for dipole fitting\n', nchans);
if nchans<3*cfg.numdipoles
  error('not enough channels to perform a dipole fit');
end

% extract the relevant potential distribution
if ~isfield(data, 'topo') & isfield(data, 'time') & isfield(data, 'avg')
  % this looks like a timelocked ERP average
  if ischar(cfg.latency)
    cfg.latency = data.time([1 end]);
  end
  tbeg = nearest(data.time, cfg.latency(1));
  tend = nearest(data.time, cfg.latency(end));
  cfg.latency = [data.time(tbeg) data.time(tend)];
  Vdata = data.avg(seldata, tbeg:tend);
else
  % this looks like a component analysis
  Vdata = data.topo(seldata,cfg.component);
end

if size(Vdata,2)==0
  error('empty selection of data');
end

ntime = size(Vdata,2);
fprintf('using %d timepoints/components for dipole fitting\n', ntime);

% check whether EEG is average referenced
if strcmp(sens.type, 'electrodes')
  if rv(Vdata, avgref(Vdata))>0.01
    warning('the EEG data is not average referenced, correcting this');
  end
  Vdata = avgref(Vdata);
end

if strcmp(cfg.model, 'regional')
  % set to zeros if no initial position was specified for the dipoles
  if ~isfield(cfg, 'dip')
    cfg.dip.pos = zeros(cfg.numdipoles, 3);
  end
  % check the input configuration
  if size(cfg.dip.pos,1)~=cfg.numdipoles
    error('inconsistent number of dipoles in configuration')
  end
elseif strcmp(cfg.model, 'moving')
  if ~isfield(cfg, 'dip')
    for t=1:ntime
      % set to zeros if no initial position was specified for the dipoles
      cfg.dip(t).pos = zeros(cfg.numdipoles, 3);
    end
  end
  for t=1:ntime
    % check the input configuration
    if size(cfg.dip(t).pos,1)~=cfg.numdipoles
      error('inconsistent number of dipoles in configuration')
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% perform the dipole scan, this is usefull for generating an initial guess 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(cfg.gridsearch, 'yes')
  % test whether we have a valid configuration for dipole scanning
  if cfg.numdipoles==1
    % this is ok
  elseif cfg.numdipoles==2 & ~isempty(cfg.constr)
    % this is also ok
  else
    error('dipole scanning is only possible for a single dipole or a symmetric dipole pair');
  end
  
  % construct the grid on which the scanning will be done
  [grid, cfg] = prepare_dipole_grid(cfg, vol, sens);
  
  if ~isempty(cfg.constr)
    % expand the number of parameters from one (3) to two dipoles (6)
    grid.pos = grid.pos(:,cfg.constr.expand) .* repmat(cfg.constr.mirror, size(grid.pos,1), 1);
  end
  for i=1:length(grid.inside)
    fprintf('scanning grid location %d/%d\n', i, length(grid.inside));
    indx = grid.inside(i);
    if isfield(grid, 'leadfield')
      % reuse the previously computed leadfield
      lf = grid.leadfield{indx};
    else
      pos = reshape(grid.pos(indx,:), [3 cfg.numdipoles])';
      lf = compute_leadfield(pos, sens, vol);
    end
    
    % the model is V=lf*mom+noise, therefore mom=pinv(lf)*V estimates the
    % dipole moment this makes the model potential U=lf*pinv(lf)*V and the
    % model error is norm(V-U) = norm(V-lf*pinv(lf)*V) = norm((eye-lf*pinv(lf))*V)
    if strcmp(cfg.model, 'regional')
      % sum over all latencies
      grid.error(indx,1) = sum(sum(((eye(nchans)-lf*pinv(lf))*Vdata).^2));
    elseif  strcmp(cfg.model, 'moving')
      % remember error for every latency independently
      grid.error(indx,:) = sum(((eye(nchans)-lf*pinv(lf))*Vdata).^2);
    end    
  end
  % find the grid point(s) with the minimum error
  [err, indx] = min(grid.error(grid.inside,:), [], 1);
  indx = grid.inside(indx);
  for i=1:length(indx)
    dip(i).pos = grid.pos(indx(i),:);
    fprintf('found minimum after scanning on grid point [%f %f %f]\n', dip(i).pos(1), dip(i).pos(2), dip(i).pos(3));
  end
else
  % use the initial guess supplied in the configuration
  dip = cfg.dip;
end

if ~isfield(dip, 'mom')
  % this must be added, otherwise there is an error in the assignment of
  % the dipole fit result later
  for t=1:length(dip)
    dip(t).mom = [];
  end
end

if strcmp(cfg.nonlinear, 'yes')
  if strcmp(cfg.model, 'regional')
    % perform the non-linear dipole fit for all latencies together
    dip = compute_dipole_fit(dip, Vdata, sens, vol, cfg.constr);
  elseif strcmp(cfg.model, 'moving')
    % perform the non-linear dipole fit for each latency independently
    for t=1:ntime
      dip(t) = compute_dipole_fit(dip(t), Vdata(:,t), sens, vol, cfg.constr);
    end
  end
end

if strcmp(cfg.model, 'regional')
  % re-compute the leadfield in order to compute the model potential and dipole moment
  lf = compute_leadfield(dip.pos, sens, vol);
  
  % compute all details of the final dipole model
  dip.mom = pinv(lf)*Vdata;
  dip.pot = lf*dip.mom;
  dip.rv  = rv(Vdata, dip.pot);
  
elseif strcmp(cfg.model, 'moving')
  for t=1:ntime
    % re-compute the leadfield in order to compute the model potential and dipole moment
    lf = compute_leadfield(dip(t).pos, sens, vol);
    
    % compute all details of the final dipole model
    dip(t).mom = pinv(lf)*Vdata(:,t);
    dip(t).pot = lf*dip(t).mom;
    dip(t).rv  = rv(Vdata(:,t), dip(t).pot);
  end % ntime
end % model

% add the version details of this function call to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: dipolefitting.m,v 1.14 2005/02/21 07:56:54 roboos Exp $';

% try to add the configuration of the input data
try, cfg.previous = data.cfg; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% collect the results
source.cfg    = cfg;
source.label  = data.label(seldata);
source.dip    = dip;
source.Vdata  = Vdata;
if isfield(data, 'time') & ~iscell(data.time)
  source.time   = data.time(nearest(data.time, cfg.latency(1)):nearest(data.time, cfg.latency(2)));
else
  source.component = cfg.component;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that helps select the proper dipole fitting subfunction and
% that catches errors due to non-convergence
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [dipout] = compute_dipole_fit(dipin, dat, sens, vol, constr)
try,
  if strcmp(sens.type, 'electrodes')
    dipout = eeg_dipole_fit(dipin, dat, sens.pnt, vol, constr);
  elseif strcmp(sens.type, 'gradiometers')
    dipout = meg_dipole_fit(dipin, dat, sens, vol, constr);
  end
catch
  dipout.pos = nan*dipin.pos;
  dipout.mom = [];
end
