function [freq] = freqanalysis_mtmconvol(cfg, data);

% FREQANALYSIS_MTMCONVOL performs time-frequency analysis on any time series trial data
% using the 'multitaper method' (MTM) based on Slepian sequences as tapers.
%
% [freq] = freqanalysis(cfg, data)
%
% The data should be organised in a structure as obtained from 
% the PREPROCESSING function. The configuration should be according to 
%
% cfg.output     = 'pow'; 
%
% If cfg.output is 'pow', then only the power spectra will be returned. 
% In that case, cfg should specify the names of channels to be analyzed in the  
% tag cfg.channel, which is a cell-array, which can contan individual channel labels
% or known groups of channels such as 'all', 'MEG', 'EEG'.
% See the CHANNELSELECTION function for more information.
% 
% Alternatively:
%
% cfg.output      = 'powandcsd';
%
% If cfg.output is 'powandcsd', then the power- and cross-spectra are returned. 
% In that case, cfg should contain the tag cfg.channelcmb, specifying the
% channel combinations:
%
% cfg.channelcmb      =[{'MZO02' 'MLP11'}; {'MZO02' 'MLP12'}; {'MZO02' 'MLP21'}];
%
% If you want to select more than just a few combinations of channels,
% e.g. all, you can use the CHANNELCOMBINATIONS helper function. Look
% at the help of that function for more options and to learn how to
% make specific channel combinations. Selecting all combinations is
% done with
%
% cfg.channelcmb   = channelcombination({'all' 'all}, data.label);
%
% You have to specify 
%
% - the frequencies. Unit is Hertz= 1./s.
% - the amoung of spectral smoothing through multi-tapering. Unit is Hertz = 1./s.
%   Be aware that 4 Hz smoothing means plus-minus 4 Hz, i.e. a 8 Hz smoothing box.
% - for each frequency separately, you have to specify the length of the time window
% - for all frequencies together, you have to specify the times on which the analysis
%    windows should be centered. Those times are relative to the data.offset. Their unit is second.
%
% Thus, possible configurations are:
%
% cfg.method          = 'mtmconvol';           
% cfg.foi             = 20:5:150;
% numfoi              = length(cfg.foi);
% cfg.t_ftimwin       = zeros(1,numfoi);
% cfg.t_ftimwin(:)    = 0.5;
% cfg.tapsmofrq       = zeros(1,numfoi);
% cfg.tapsmofrq(:)    = 10;
% cfg.toi             = 1:0.1:1.2; 
% 
% Furthermore, you have to specify, whether you want to obtain the average spectra: 
%
% cfg.keeptrials  = 'no';
% cfg.keeptapers  = 'no';               
%
% or whether you want to keep the spectra of the individual trial and/or tapers:
% 
% cfg.keeptrials  = 'yes';
% cfg.keeptapers  = 'yes';               
% 
% And finally, you have to specify a length in seconds to which the data can be padded out.  
% cfg.pad         = 2;        
% 
% You can also ask the routine to use the maximum existing data length:
% cfg.pad         = 'maxperlen';         
% Be aware that this will run very slow if you specify cfg.pad as
% maxperlen AND the number of samples turns out to have a large prime factor sum. This is because
% MTMFFTs will then be computed very inefficiently.

% undocumented experimental options
%   cfg.taper = 'dpss'    use discrete prolate spheroidal sequences (Slepian sequences)
%   cfg.taper = 'hanning' use single Hanning taper

% Copyright (c) 2003,2004 F.C. Donders Centre
%
% $Log: freqanalysis_mtmconvol.m,v $
% Revision 1.11  2005/02/16 09:21:24  jansch
% inserted the cfg.output-option 'powandfourier'. in addition to outputting
% the powerspectrum, the complex discrete fourier spectrum is returned,
% allowing for computation of phases.
%
% Revision 1.10  2005/01/19 08:42:42  jansch
% removed obsolete code for generating and checking channelcombinations
% cleaned up handling of sgnindx/sgncmbindx
% ensured that power is computed for all channels that are in channelcmb
%
% Revision 1.9  2005/01/18 15:11:38  roboos
% Cleaned up configuration for sgn/sgncmb, now exclusively using channel/channelcmb which is consistent with rest of fieldtrip and freqanalysis documentation. Also the output now only contains freq.label/labelcmb and not any more the old sgn/sgncmb.
%
% Revision 1.8  2005/01/17 14:51:16  roboos
% implemented tapering with single window according to user specification
% (cfg.taper) to allow hanning and other windows to be used easily.
% The default is to multitaper with a dpss sequence.
%
% Revision 1.7  2004/12/20 14:52:56  roboos
% changed rounding off of timboi
%
% Revision 1.6  2004/12/20 14:47:03  roboos
% fixed rounding off error in sample number timboi
%
% Revision 1.5  2004/12/20 13:03:14  jansch
% fixed bug that incorrectly handled missing data as zeros
%
% Revision 1.4  2004/11/18 16:51:58  roboos
% updated the help: pointed to CHANNELCOMBINATION for cfg.channelcmb
%
% Revision 1.3  2004/10/28 09:00:33  roboos
% fixed bug in caller function detection (for matlab 6.1 and 6.5)
%
% Revision 1.2  2004/10/28 07:21:46  roboos
% added check to ensure that the FREQANALYSIS wrapper is started instead of the
% FERQANALYSIS_xxx subfunctions
%
% Revision 1.1  2004/09/21 12:04:02  marsie
% the mtmconvol method of multitaperanalysis.m has been moved to this seperate function
%
% Revision 1.3  2004/09/16 11:40:34  marsie
% fixed bug in 'mtmfft' that slowed computation down w/o causing bad results
%
% Revision 1.2  2004/08/31 14:36:30  marsie
% added support for data segments, which are to short to place a taper
%
% Revision 1.1  2004/08/25 17:38:28  marsie
% initial CVS commit
% this version is based on the original freqanalysis including the following changes
% and improvements:
% - the DFT method is no longer supported
% - methods are renamed with the prefix 'mtm': 'mtmfft' and 'mtmconvol'
% - the last taper returned by 'dpss' is not used
% - the function checks for the consistency of data length and spectral smoothing
% - improved performance by vectorization of loops
% - output of 'mtmfft' is correctly dimensioned for single frequency output
%

% ensure that this function is started as a subfunction of the FREQANALYSIS wrapper
[s, i] = dbstack;
if length(s)>1
  [caller_path, caller_name, caller_ext] = fileparts(s(2).name);
else
  caller_path = '';
  caller_name = '';
  caller_ext  = '';
end
% evalin('caller', 'mfilename') does not work for Matlab 6.1 and 6.5
if ~strcmp(caller_name, 'freqanalysis')
  error(['you should call FREQANALYSIS, instead of the ' upper(mfilename) ' subfunction']);
end

% set all the defaults
if ~isfield(cfg, 'method'),        cfg.method     = 'mtmconvol';  end
if ~isfield(cfg, 'keeptapers'),    cfg.keeptapers = 'no';   end
if ~isfield(cfg, 'keeptrials'),    cfg.keeptrials = 'no';   end
if ~isfield(cfg, 'output'),        cfg.output     = 'powandcsd';  end
if ~isfield(cfg, 'pad'),           cfg.pad        = 'maxperlen';  end
if ~isfield(cfg, 'taper'),         cfg.taper      = 'dpss';       end

% setting a flag (csdflg) that determines whether this routine outputs 
% only power-spectra or power-spectra and cross-spectra?
if strcmp(cfg.output,'pow')  
  csdflg = 0;
  fftflg = 0;
elseif strcmp(cfg.output,'powandcsd')
  csdflg = 1;
  fftflg = 0;
elseif strcmp(cfg.output,'powandfourier')
  csdflg = 0;
  fftflg = 1;
end

% determine the corresponding indices of all channels
sgnindx     = match_str(data.label, cfg.channel);
numsgn      = size(sgnindx,1);
if csdflg
  % determine the corresponding indices of all channel combinations
  for k=1:size(cfg.channelcmb,1) 
    sgncmbindx(k,1) = strmatch(cfg.channelcmb(k,1), data.label, 'exact');
    sgncmbindx(k,2) = strmatch(cfg.channelcmb(k,2), data.label, 'exact');
  end
  
  numsgncmb   = size(sgncmbindx,1);
  sgnindx     = unique([sgnindx(:); sgncmbindx(:)]);
  numsgn      = length(sgnindx);
  
  cutdatindcmb = zeros(size(sgncmbindx));
  for sgnlop = 1:numsgn
    cutdatindcmb(find(sgncmbindx == sgnindx(sgnlop))) = sgnlop;
  end    
end

% if rectan is 1 it means that trials are of equal lengths
numper      = size(data.trial,2);
rectan = 1; 
for perlop = 1:numper
  numdatbnsarr(perlop,1) = size(data.trial{perlop},2);
  if numdatbnsarr(perlop,1) ~= numdatbnsarr(1,1)
    rectan = 0;
  end
end

%if cfg.pad is 'maxperlen', this is realized here:
if ischar(cfg.pad) 
  if strcmp(cfg.pad,'maxperlen')
    cfg.pad = max(numdatbnsarr,[],1) ./ data.fsample;
  end
end
numsmp = cfg.pad .* data.fsample;

% keeping trials and/or tapers?
if strcmp(cfg.keeptrials,'no') &  strcmp(cfg.keeptapers,'yes')
  error('There is currently no support for keeping tapers WITHOUT KEEPING TRIALS.');
end
if strcmp(cfg.keeptrials,'no') &  strcmp(cfg.keeptapers,'no')
  keep = 1;
elseif strcmp(cfg.keeptrials,'yes') &  strcmp(cfg.keeptapers,'no')
  keep = 2;
  % elseif strcmp(cfg.keeptrials,'no') &  strcmp(cfg.keeptapers,'yes')
  %     keep = 3;
elseif strcmp(cfg.keeptrials,'yes') &  strcmp(cfg.keeptapers,'yes')
  keep = 4;
end

% doing the computation for the different methods MTMFFT, MTMCONVOL
if strcmp(cfg.method,'mtmconvol')
  if strcmp(cfg.keeptrials,'yes') &  strcmp(cfg.keeptapers,'yes')
    error('Currently no support for keeping trials AND tapers when using MTMCONVOL as the method.');
  end
  minoffset = min(data.offset);
  timboi = round(cfg.toi .* data.fsample - minoffset);
  toi = round(cfg.toi .* data.fsample) ./ data.fsample; 
  numtoi = length(toi);
  numfoi = length(cfg.foi);
  knlspctrmstr = cell(numfoi,1);
  for foilop = 1:numfoi
    acttapnumsmp = round(cfg.t_ftimwin(foilop) .* data.fsample);
    if strcmp(cfg.taper, 'dpss')
      % create a sequence of DPSS (Slepian) tapers
      tap = dpss(acttapnumsmp, acttapnumsmp .* (cfg.tapsmofrq(foilop)./data.fsample));
    else
      % create a single taper according to the window specification as a
      % replacement for the DPSS (Slepian) sequence
      tap = window(cfg.taper, acttapnumsmp)';
      % freqanalysis_mtmfft always throws away the last taper of the Slepian
      % sequence, so add a dummy taper
      tap(2,:) = nan;
    end
    numtap(foilop) = size(tap,2)-1;
    if (numtap(foilop) < 1)
      error(sprintf(...
        '%.3f Hz : datalength to short for specified smoothing\ndatalength: %.3f s, smoothing: %.3f Hz, minimum smoothing: %.3f Hz',...
        cfg.foi(foilop), acttapnumsmp/data.fsample, cfg.tapsmofrq(foilop), data.fsample/acttapnumsmp));
    elseif (numtap(foilop) < 2)
      fprintf('%.3f Hz : WARNING - using only one taper for specified smoothing\n',cfg.foi(foilop));
    end
    ins = ceil(numsmp./2) - floor(acttapnumsmp./2);
    prezer = zeros(ins,1);
    pstzer = zeros(numsmp - ((ins-1) + acttapnumsmp)-1,1);
    ind = (0:acttapnumsmp-1)' .* ...
      ((2.*pi./data.fsample) .* cfg.foi(foilop));
    knlspctrmstr{foilop} = complex(zeros(numtap(foilop),numsmp));
    for taplop = 1:numtap(foilop)
      try
        knlspctrmstr{foilop}(taplop,:) = ...
          fft(complex(vertcat(prezer,tap(:,taplop).*cos(ind),pstzer), ...
          vertcat(prezer,tap(:,taplop).*sin(ind),pstzer)),[],1); 
      end
    end
  end    
  if keep == 1
    powspctrm = zeros(numsgn,numfoi,numtoi);
    if csdflg, crsspctrm     = complex(zeros(numsgncmb,numfoi,numtoi)); end
    if fftflg, fourierspctrm = complex(zeros(numsgn,numfoi,numtoi));    end
    cntpertoi = zeros(numfoi,numtoi);
    dimord    = 'sgncmb_frq_tim';
  elseif keep == 2
    powspctrm = zeros(numper,numsgn,numfoi,numtoi);
    if csdflg, crsspctrm     = complex(zeros(numper,numsgncmb,numfoi,numtoi)); end
    if fftflg, fourierspctrm = complex(zeros(numper,numsgn,numfoi,numtoi));    end
    dimord    = 'rpt_sgncmb_frq_tim';
  elseif keep == 4
    powspctrm = zeros(1,numsgn,numfoi,numtoi);            
    if csdflg, crsspctrm     = complex(zeros(1,numsgncmb,numfoi,numtoi)); end
    if fftflg, fourierspctrm = complex(zeros(1,numsgn,numfoi,numtoi));    end
    cnt = 0;
    dimord    = 'rpttap_sgncmb_frq_tim';
  end
  for perlop = 1:numper
    fprintf('processing trial %d: %d samples\n', perlop, numdatbnsarr(perlop,1));
    if keep == 2
      cnt = perlop;
    end
    numdatbns = numdatbnsarr(perlop,1);  
    prepad = zeros(1,data.offset(perlop) - minoffset);
    pstpad = zeros(1,minoffset + numsmp - (data.offset(perlop) + numdatbns));
    datspctra = complex(zeros(numsgn,numsmp));
    for sgnlop = 1:numsgn
      datspctra(sgnlop,:) = fft([prepad, data.trial{perlop}(sgnindx(sgnlop),:), ...
          pstpad],[],2);
    end    
    for foilop = 1:numfoi
      fprintf('processing frequency %d (%.2f Hz), %d tapers\n', foilop,cfg.foi(foilop),numtap(foilop));
      actfoinumsmp = cfg.t_ftimwin(foilop) .* data.fsample; 
      acttimboiind = ...
        find(timboi >= (-minoffset + data.offset(perlop) + (actfoinumsmp ./ 2)) & ...
        timboi < (-minoffset + data.offset(perlop) + numdatbns - (actfoinumsmp ./2)));
      nonacttimboiind = ...
        find(timboi < (-minoffset + data.offset(perlop) + (actfoinumsmp ./ 2)) | ...
        timboi >= (-minoffset + data.offset(perlop) + numdatbns - (actfoinumsmp ./2)));
      acttimboi = timboi(acttimboiind);
      numacttimboi = length(acttimboi);
      if keep ==1
        cntpertoi(foilop,acttimboiind) = cntpertoi(foilop,acttimboiind) + 1;
      end
      for taplop = 1:numtap(foilop)
        if keep == 3
          cnt = taplop;
        elseif keep == 4
          cnt = cnt+1;
        end
        autspctrmacttap = complex(zeros(numsgn,numacttimboi));
        if numacttimboi > 0
          for sgnlop = 1:numsgn
            dum = fftshift(ifft(datspctra(sgnlop,:) .* ...
              [knlspctrmstr{foilop}(taplop,:)],[],2));
            autspctrmacttap(sgnlop,:) = dum(acttimboi);
          end 
          powdum = 2.* abs(autspctrmacttap) .^ 2 ./ actfoinumsmp;
        end
        if keep == 1 & numacttimboi > 0
          powspctrm(:,foilop,acttimboiind) = powspctrm(:,foilop,acttimboiind) + ...
            reshape(powdum ./ numtap(foilop),[numsgn,1,numacttimboi]);
        elseif keep == 2 & numacttimboi > 0
          powspctrm(cnt,:,foilop,acttimboiind) = powspctrm(cnt,:,foilop,acttimboiind) + ...
            reshape(powdum ./ numtap(foilop),[1,numsgn,1,numacttimboi]);
          powspctrm(cnt,:,foilop,nonacttimboiind) = nan;
        elseif keep == 4 & numacttimboi > 0
          powspctrm(cnt,:,foilop,acttimboiind) = ...
            reshape(powdum,[1,numsgn,1,numacttimboi]);
          powspctrm(cnt,:,foilop,nonacttimboiind) = nan;
        elseif (keep == 4 | keep == 2) & numacttimboi == 0
          powspctrm(cnt,:,foilop,nonacttimboiind) = nan;
        end
        if fftflg
          fourierdum = 2 .* (autspctrmacttap) ./ actfoinumsmp; %cf Numercial Receipes 13.4.9
          if keep == 1 & numacttimboi > 0
            fourierspctrm(:,foilop,acttimboiind) = ...
              fourierspctrm(:,foilop,acttimboiind) + ...
              reshape((fourierdum ./ numtap(foilop)),[numsgn,1,numacttimboi]);
          elseif keep == 2 & numacttimboi > 0 
            fourierspctrm(cnt,:,foilop,acttimboiind) = ...
              fourierspctrm(cnt,:,foilop,acttimboiind) + ...
              reshape(fourierdum ./ numtap(foilop),[1,numsgn,1,numacttimboi]);
            fourierspctrm(cnt,:,foilop,nonacttimboiind) = nan;
          elseif keep == 4 & numacttimboi > 0
            fourierspctrm(cnt,:,foilop,acttimboiind) = ...
              reshape(fourierdum,[1,numsgn,1,numacttimboi]);
            fourierspctrm(cnt,:,foilop,nonacttimboiind) = nan;
          elseif (keep == 4 | keep == 2) & numacttimboi == 0
            fourierspctrm(cnt,:,foilop,nonacttimboiind) = nan; 
          end
        end
        if csdflg
          csddum = 2.* (autspctrmacttap(cutdatindcmb(:,1),:) .* ...
            conj(autspctrmacttap(cutdatindcmb(:,2),:))) ./ actfoinumsmp;
          if keep == 1 & numacttimboi > 0
            crsspctrm(:,foilop,acttimboiind) = ...
              crsspctrm(:,foilop,acttimboiind) + ...
              reshape((csddum ./ numtap(foilop)),[numsgncmb,1,numacttimboi]);
          elseif keep == 2 & numacttimboi > 0
            crsspctrm(cnt,:,foilop,acttimboiind) = ...
              crsspctrm(cnt,:,foilop,acttimboiind) + ...
              reshape(csddum ./ numtap(foilop),[1,numsgncmb,1,numacttimboi]);
            crsspctrm(cnt,:,foilop,nonacttimboiind) = nan;
          elseif keep == 4 & numacttimboi > 0
            crsspctrm(cnt,:,foilop,acttimboiind) = ...
              reshape(csddum,[1,numsgncmb,1,numacttimboi]);
            crsspctrm(cnt,:,foilop,nonacttimboiind) = nan;
          elseif (keep == 4 | keep == 2) & numacttimboi == 0
            crsspctrm(cnt,:,foilop,nonacttimboiind) = nan;
          end
        end
      end%of taplop
    end% of foilop    
  end%of perlop    
  if keep == 1
    warning off
    powspctrm(:,:,:) = powspctrm(:,:,:) ./ repmat(permute(cntpertoi,[3,1,2]),[numsgn,1,1]);
    if fftflg
      fourierspctrm(:,:,:) = fourierspctrm(:,:,:) ./ repmat(permute(cntpertoi,[3,1,2]),[numsgn,1,1]);
    end
    if csdflg
      crsspctrm(:,:,:) = crsspctrm(:,:,:) ./ repmat(permute(cntpertoi,[3,1,2]),[numsgncmb,1,1]);
    end
    warning on
  end
end 

% add information about the version of this function to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i1] = dbstack;
  cfg.version.name = st(i1);
end
cfg.version.id = '$Id: freqanalysis_mtmconvol.m,v 1.11 2005/02/16 09:21:24 jansch Exp $';
% remember the configuration details of the input data
cfg.previous   = data.cfg;

% collect the results
freq.label      = data.label(sgnindx);
freq.dimord     = dimord;
freq.powspctrm  = powspctrm;
freq.foi        = cfg.foi;
freq.toi        = toi;

if csdflg 
  freq.labelcmb   = cfg.channelcmb;
  freq.crsspctrm  = crsspctrm;
end

if fftflg
  freq.fourierspctrm  = fourierspctrm;
end

try freq.grad = data.grad; end   % remember the gradiometer array
try freq.elec = data.elec; end   % remember the electrode array

% remember the full configuration details
freq.cfg          = cfg;
