function multiplotTFR(cfg, data)

% MULTIPLOTTFR plots time-frequency representations in a toporgaphical layout.
% The plots of the indivual sensors are arranged according to their location
% specified in the layout file.
%
% Use as
%   multiplotTFR(cfg, freq)
% where freq is the result from FREQANALYSIS and the cfg structure contains
% all configurations details for the plotting:
%
% cfg.layout           = specification of the layout, see below
% cfg.baseline         = 'yes','no' or [time1 time2]   (default = 'no')
% cfg.baselinetype     = 'absolute' or 'relative'      (default = 'absolute')
% cfg.xlim             = 'maxmin' or [xmin xmax]       (default = 'maxmin')
% cfg.ylim             = 'maxmin' or [ymin ymax]       (default = 'maxmin')
% cfg.zlim             = 'maxmin','absmax' or [zmin zmax] (default = 'maxmin')
% cfg.comment          = string of text                (default = date + zlimits)
%                        Add 'comment' to graph (according to COMNT in the layout)
% cfg.showlabels       = 'yes', 'no', 'numbers'        (default = 'no')
% cfg.fontsize         = fontsize of text              (default = 6)
% cfg.cohtargetchannel = string with the channel relative to which coherence
%                        is plotted (default is to plot power)
%
% The layout defines how the channels are arranged and what the size of each
% subplot is. You can specify the layout in a variety of ways:
%  - you can give the name of an ascii layout file with extension *.lay
%  - you can give the name of an electrode file
%  - you can give an electrode definition, i.e. "elec" structure
%  - you can give a gradiometer definition, i.e. "grad" structure
% If you do not specify any of these and the data structure contains an
% electrode or gradiometer structure (common for MEG data, since the header
% of the MEG datafile contains the gradiometer information), that will be
% used for creating a layout. If you want to have more fine-grained control
% over the layout of the subplots, you should create your own layout file.
%
% See also TOPOPLOTTFR, MULTIPLOTER, TOPOPLOTTFR

% Copyright (C) 2003, Ole Jensen
%
% $Log: multiplotTFR.m,v $
% Revision 1.5  2005/02/07 17:12:00  roboos
% changed handling of layout files (using new function createlayout), now also supports automatic layout creation based on gradiometer/electrode definition in data, updated help, cleaned up indentation
%
% Revision 1.4  2004/09/24 15:54:54  roboos
% included the suggested improvements by Doug Davidson: added option cfg.cohtargetchannel
% and updated the help
%
% Revision 1.3  2004/09/01 18:02:23  roboos
% added copyright statements, removed cfg as output argument
%

% set the defaults
if ~isfield(cfg,'baseline'),        cfg.baseline   = 'no'; end;
if ~isfield(cfg,'baselinetype'),    cfg.baselinetype = 'absolute'; end;
if ~isfield(cfg,'xlim'),            cfg.xlim       = 'maxmin'; end;
if ~isfield(cfg,'ylim'),            cfg.ylim       = 'maxmin'; end;
if ~isfield(cfg,'zlim'),            cfg.zlim       = 'maxmin'; end;
if ~isfield(cfg,'comment'),         cfg.comment    = strcat([date '\n']); end;
if ~isfield(cfg,'showlabels'),      cfg.showlabels = 'no'; end;
if ~isfield(cfg,'channel'),         cfg.channel    = 'all'; end;
if ~isfield(cfg,'fontsize'),        cfg.fontsize   = 6; end;

% Option for plotting cohspctrm (doudav)
if isfield(cfg,'cohtargetchannel'), data=pwrspctm2cohspctrm(cfg,data); end;

% read or create the layout that will be used for plotting
if isfield(cfg, 'layout')
  lay = createlayout(cfg.layout);               % use specified layout
elseif isfield(cfg, 'layoutname')
  lay = createlayout(cfg.layoutname);           % idem, for backward compatibility
elseif isfield(data,  'grad')
  lay = createlayout(data.grad);                % create layout from gradiometer definition
elseif isfield(data, 'elec')
  lay = createlayout(data.elec);                % create layout from electrode definition
else
  lay = createlayout('CTF151s.lay');            % revert to the FCDC default
end

% convert the layout to Ole's style of variable names
X      = lay.prj(:,1);
Y      = lay.prj(:,2);
Width  = lay.box(:,1);
Height = lay.box(:,2);
Lbl    = lay.label;

TFR = data.powspctrm;

if ~strcmp(cfg.baseline,'no')
  if strcmp(cfg.baseline,'yes')
    cfg.baseline = [];
    cfg.baseline(1) = min(data.toi);
    cfg.baseline(2) = 0;
  end

  tidx = find(data.toi >= cfg.baseline(1) & data.toi <= cfg.baseline(2));

  for k=1:size(TFR,2)
    for l=1:size(TFR,1)
      TFRbl(l,k) = squeeze(nan_mean(TFR(l,k,tidx),3));
    end
  end


  for k=1:size(TFR,2)
    for l=1:size(TFR,1)
      if strcmp(cfg.baselinetype,'absolute')
        TFR(l,k,:) = TFR(l,k,:) - TFRbl(l,k);
      elseif strcmp(cfg.baselinetype,'relative')
        TFR(l,k,:) = 100*(TFR(l,k,:) - TFRbl(l,k))./TFRbl(l,k);
      end
    end
  end
end

if strcmp(cfg.xlim,'maxmin')
  xmin = min(data.toi);
  xmax = max(data.toi);
else
  xmin = cfg.xlim(1);
  xmax = cfg.xlim(2);
end

if strcmp(cfg.ylim,'maxmin')
  ymin = min(data.foi);
  ymax = max(data.foi);
else
  ymin = cfg.ylim(1);
  ymax = cfg.ylim(2);
end

l = [];
for k=1:length(Lbl)
  if ~isempty(Lbl(k))
    l = [l cellstrmatch(Lbl(k),data.label)];
  end
end

if strcmp(cfg.zlim,'maxmin')
  zmin = min(min(min(TFR(l,:,:))));
  zmax = max(max(max(TFR(l,:,:))));
elseif strcmp(cfg.zlim,'absmax')
  mval = max([abs(min(min(min(TFR(l,:,:))))) abs(max(max(max(TFR(l,:,:))))) ]);
  zmin = - mval;
  zmax = mval;
else
  zmin = cfg.zlim(1);
  zmax = cfg.zlim(2);
end

xidc = find(data.toi >= xmin & data.toi <= xmax);
yidc = find(data.foi >= ymin & data.foi <= ymax);

data.toi = data.toi(xidc);
data.foi =  data.foi(yidc);
TFR = TFR(:,yidc,xidc);

xScaleFac = 1/(max(Width)+ max(X) - min(X));
yScaleFac = 1/(max(Height)+ max(Y) - min(Y));

Xpos = xScaleFac*(X-min(X));
Ypos = 0.9*yScaleFac*(Y-min(Y));

%hold on

for k=1:length(Lbl)
  l = cellstrmatch(Lbl(k),data.label);
  if ~isempty(l)
    subplotOL('position',[Xpos(k) Ypos(k)+(Height(k)*yScaleFac) Width(k)*xScaleFac Height(k)*yScaleFac])
    % plot([0 0],[1 1])
    imagesc(data.toi,data.foi,squeeze(TFR(l,:,:)),[zmin zmax]);axis xy
    set(gca,'YTickLabel',[])
    set(gca,'XTickLabel',[])

    if strcmp(cfg.showlabels,'yes')
      title(Lbl(k),'Fontsize',cfg.fontsize,'VerticalAlignment','middle')
    end

    if strcmp(cfg.showlabels,'numbers')
      title(l,'Fontsize',cfg.fontsize,'VerticalAlignment','middle')
    end
  end
end


% title(sprintf('Zlimits: \n %0.2e to %0.2e ',zmin,zmax));

l = cellstrmatch('COMNT',Lbl);
if ~isempty(l)
  cfg.comment = [cfg.comment  sprintf('Zlim: %0.2e to %0.2e',zmin,zmax)];


  subplotOL('position',[Xpos(l) Ypos(l)+(Height(l)*yScaleFac) Width(l)*xScaleFac Height(l)*yScaleFac])
  text(0,0,sprintf(cfg.comment),'Fontsize',cfg.fontsize);
  axis off
end

l = cellstrmatch('SCALE',Lbl);
if ~isempty(l)
  subplotOL('position',[Xpos(l) Ypos(l)+(Height(l)*yScaleFac) Width(l)*xScaleFac Height(l)*yScaleFac])
  imagesc(data.toi,data.foi,randn(length(data.toi),length(data.foi)));axis xy
  set(gca,'Fontsize',cfg.fontsize)

end

axis tight
orient landscape
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function theAxis = subplotOL(nrows, ncols, thisPlot)

%SUBPLOT Create axes in tiled positions.
%   H = SUBPLOT(m,n,p), or SUBPLOT(mnp), breaks the Figure window
%   into an m-by-n matrix of small axes, selects the p-th axes for
%   for the current plot, and returns the axis handle.  The axes
%   are counted along the top row of the Figure window, then the
%   second row, etc.  For example,
%
%       SUBPLOT(2,1,1), PLOT(income)
%       SUBPLOT(2,1,2), PLOT(outgo)
%
%   plots income on the top half of the window and outgo on the
%   bottom half.
%
%   SUBPLOT(m,n,p), if the axis already exists, makes it current.
%   SUBPLOT(H), where H is an axis handle, is another way of making
%   an axis current for subsequent plotting commands.
%
%   SUBPLOT('position',[left bottom width height]) creates an
%   axis at the specified position in normalized coordinates (in
%   in the range from 0.0 to 1.0).
%
%   If a SUBPLOT specification causes a new axis to overlap an
%   existing axis, the existing axis is deleted.  For example,
%   the statement SUBPLOT(1,2,1) deletes all existing axes overlapping
%   the left side of the Figure window and creates a new axis on that
%   side.
%
%   SUBPLOT(111) is an exception to the rules above, and is not
%   identical in behavior to SUBPLOT(1,1,1).  For reasons of backwards
%   compatibility, it is a special case of subplot which does not
%   immediately create an axes, but instead sets up the figure so that
%   the next graphics command executes CLF RESET in the figure
%   (deleting all children of the figure), and creates a new axes in
%   the default position.  This syntax does not return a handle, so it
%   is an error to specify a return argument.  The delayed CLF RESET
%   is accomplished by setting the figure's NextPlot to 'replace'.

%   Copyright (c) 1984-98 by The MathWorks, Inc.
%   $Revision: 1.5 $  $Date: 2005/02/07 17:12:00 $

% we will kill all overlapping axes siblings if we encounter the mnp
% or m,n,p specifier (excluding '111').
% But if we get the 'position' or H specifier, we won't check for and
% delete overlapping siblings:
narg = nargin;
kill_siblings = 0;
create_axis = 1;
delay_destroy = 0;
tol = sqrt(eps);
if narg == 0 % make compatible with 3.5, i.e. subplot == subplot(111)
  nrows = 111;
  narg = 1;
end

%check for encoded format
handle = '';
position = '';
if narg == 1
  % The argument could be one of 3 things:
  % 1) a 3-digit number 100 < num < 1000, of the format mnp
  % 2) a 3-character string containing a number as above
  % 3) an axis handle
  code = nrows;

  % turn string into a number:
  if(isstr(code)) code = str2double(code); end

  % number with a fractional part can only be an identifier:
  if(rem(code,1) > 0)
    handle = code;
    if ~strcmp(get(handle,'type'),'axes')
      error('Requires valid axes handle for input.')
    end
    create_axis = 0;
    % all other numbers will be converted to mnp format:
  else
    thisPlot = rem(code, 10);
    ncols = rem( fix(code-thisPlot)/10,10);
    nrows = fix(code/100);
    if nrows*ncols < thisPlot
      error('Index exceeds number of subplots.');
    end
    kill_siblings = 1;
    if(code == 111)
      create_axis   = 0;
      delay_destroy = 1;
    else
      create_axis   = 1;
      delay_destroy = 0;
    end
  end
elseif narg == 2
  % The arguments MUST be the string 'position' and a 4-element vector:
  if(strcmp(lower(nrows), 'position'))
    pos_size = size(ncols);
    if(pos_size(1) * pos_size(2) == 4)
      position = ncols;
    else
      error(['subplot(''position'',',...
        ' [left bottom width height]) is what works'])
    end
  else
    error('Unknown command option')
  end
  kill_siblings = 0; % Kill overlaps here also.
elseif narg == 3
  % passed in subplot(m,n,p) -- we should kill overlaps
  % here too:
  kill_siblings = 1;
end

% if we recovered an identifier earlier, use it:
if(~isempty(handle))
  set(get(0,'CurrentFigure'),'CurrentAxes',handle);
  % if we haven't recovered position yet, generate it from mnp info:
elseif(isempty(position))
  if (min(thisPlot) < 1)
    error('Illegal plot number.')
  elseif (max(thisPlot) > ncols*nrows)
    error('Index exceeds number of subplots.')
  else
    % This is the percent offset from the subplot grid of the plotbox.
    PERC_OFFSET_L = 2*0.09;
    PERC_OFFSET_R = 2*0.045;
    PERC_OFFSET_B = PERC_OFFSET_L;
    PERC_OFFSET_T = PERC_OFFSET_R;
    if nrows > 2
      PERC_OFFSET_T = 0.9*PERC_OFFSET_T;
      PERC_OFFSET_B = 0.9*PERC_OFFSET_B;
    end
    if ncols > 2
      PERC_OFFSET_L = 0.9*PERC_OFFSET_L;
      PERC_OFFSET_R = 0.9*PERC_OFFSET_R;
    end

    row = (nrows-1) -fix((thisPlot-1)/ncols);
    col = rem (thisPlot-1, ncols);

    % For this to work the default axes position must be in normalized coordinates
    if ~strcmp(get(gcf,'defaultaxesunits'),'normalized')
      warning('DefaultAxesUnits not normalized.')
      tmp = axes;
      set(axes,'units','normalized')
      def_pos = get(tmp,'position');
      delete(tmp)
    else
      def_pos = get(gcf,'DefaultAxesPosition');
    end
    col_offset = def_pos(3)*(PERC_OFFSET_L+PERC_OFFSET_R)/ ...
      (ncols-PERC_OFFSET_L-PERC_OFFSET_R);
    row_offset = def_pos(4)*(PERC_OFFSET_B+PERC_OFFSET_T)/ ...
      (nrows-PERC_OFFSET_B-PERC_OFFSET_T);
    totalwidth = def_pos(3) + col_offset;
    totalheight = def_pos(4) + row_offset;
    width = totalwidth/ncols*(max(col)-min(col)+1)-col_offset;
    height = totalheight/nrows*(max(row)-min(row)+1)-row_offset;
    position = [def_pos(1)+min(col)*totalwidth/ncols ...
      def_pos(2)+min(row)*totalheight/nrows ...
      width height];
    if width <= 0.5*totalwidth/ncols
      position(1) = def_pos(1)+min(col)*(def_pos(3)/ncols);
      position(3) = 0.7*(def_pos(3)/ncols)*(max(col)-min(col)+1);
    end
    if height <= 0.5*totalheight/nrows
      position(2) = def_pos(2)+min(row)*(def_pos(4)/nrows);
      position(4) = 0.7*(def_pos(4)/nrows)*(max(row)-min(row)+1);
    end
  end
end

% kill overlapping siblings if mnp specifier was used:
nextstate = get(gcf,'nextplot');
if strncmp(nextstate,'replace',7), nextstate = 'add'; end
if(kill_siblings)
  if delay_destroy
    if nargout
      error('Function called with too many output arguments')
    else
      set(gcf,'NextPlot','replace'); return,
    end
  end
  sibs = datachildren(gcf);
  got_one = 0;
  for i = 1:length(sibs)
    % Be aware that handles in this list might be destroyed before
    % we get to them, because of other objects' DeleteFcn callbacks...
    if(ishandle(sibs(i)) & strcmp(get(sibs(i),'Type'),'axes'))
      units = get(sibs(i),'Units');
      set(sibs(i),'Units','normalized')
      sibpos = get(sibs(i),'Position');
      set(sibs(i),'Units',units);
      intersect = 1;
      if(     (position(1) >= sibpos(1) + sibpos(3)-tol) | ...
          (sibpos(1) >= position(1) + position(3)-tol) | ...
          (position(2) >= sibpos(2) + sibpos(4)-tol) | ...
          (sibpos(2) >= position(2) + position(4)-tol))
        intersect = 0;
      end
      if intersect
        if got_one | any(abs(sibpos - position) > tol)
          delete(sibs(i));
        else
          got_one = 1;
          set(gcf,'CurrentAxes',sibs(i));
          if strcmp(nextstate,'new')
            create_axis = 1;
          else
            create_axis = 0;
          end
        end
      end
    end
  end
  set(gcf,'NextPlot',nextstate);
end

% create the axis:
if create_axis
  if strcmp(nextstate,'new'), figure, end
  ax = axes('units','normal','Position', position);
  set(ax,'units',get(gcf,'defaultaxesunits'))
else
  ax = gca;
end

% return identifier, if requested:
if(nargout > 0)
  theAxis = ax;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function l = cellstrmatch(str,strlist)
l = [];
for k=1:length(strlist)
  if strcmp(char(str),char(strlist(k)))
    l = [l k];
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function z = pwrspctm2cohspctrm(cfg, freq)
% This creates a copy of freq with new entries in the freq.pwrspctrm. The
% TFR corresponding to cfg.cohtargetchannel is set to ones, but all other
% channels are replaced with the coherence TFRs from freq.cohspctrm.
%
% [z] = pwrspctrm2cohspctrm(cfg, freq);
% 
% freq should be organised in a structure as obtained from 
% the freqdescriptives function. 
% 
% doudav; 24.09.04
z = freq;
[a a1] = match_str( cfg.cohtargetchannel, z.label);
[a b1] = match_str( cfg.cohtargetchannel, z.labelcmb(:,1));
[a b2] = match_str( cfg.cohtargetchannel, z.labelcmb(:,2));
[a c1] = match_str( z.labelcmb(b1,2), z.label);
[a c2] = match_str( z.labelcmb(b2,1), z.label);
targets  = [c1; c2];
targets2 = [b1; b2];
z.powspctrm(a1,:,:) = ones(size(z.powspctrm(a1,:,:)));
for i=1:size(targets,1),
  z.powspctrm(targets(i),:,:) = squeeze(z.cohspctrm(targets2(i),:,:));
end;
clear a a1 b1 b2 c1 c2;
