function [lf] = eeg_leadfieldb(pos, elc, vol)

% EEG_LEADFIELDB computes the electric leadfield for a dipole in a volume
% using the boundary element method
%
% [lf] = eeg_leadfieldb(pos, elc, vol)
%
% with the input arguments
%   pos		position dipole (1x3 or Nx3)
%   elc		position electrodes (optional, can be empty)
%   vol		volume conductor model
%
% the volume conductor model is a structure and should have the fields
%   vol.bnd	structure array with vertices and triangles of each boundary
%   vol.cond	conductivity of all compartments
%   vol.mat 	system matrix, which can include the electrode interpolation
%   vol.tra 	transfer matrix for interpolation from vertices to electrodes
%
% the compartment boundaries are described by a structure array with
%   vol.bnd(i).pnt
%   vol.bnd(i).pnt

% Copyright (C) 2003, Robert Oostenveld
%
% $Log: eeg_leadfieldb.m,v $
% Revision 1.1  2003/10/07 08:40:22  roberto
% made separate function for BEM, based on part of eeg_leadfield.m
%

% determine the number of compartments
ncmp = length(vol.cond);

% do some tests for the correctness of the model
if ~isfield(vol, 'bnd')
  error('there are no compartment boundaries present');
end

if length(vol.bnd)~=ncmp
  error('the number of compartments in the volume in ambiguous');
end

if ~isfield(vol, 'mat')
  error('there is no BEM system matrix present');
end

if isfield(vol, 'tra') & size(vol.tra,2)~=size(vol.mat,1)
  error('ambiguity between electrode transfer matrix and system matrix');
end

if isfield(vol, 'tra') & isempty(elc)
  warning('no electrodes specified');
elseif  isfield(vol, 'tra') & ~isempty(elc) & size(vol.tra,1)~=size(elc,1)
  error('ambiguity between number of electrodes and transfer matrix');
elseif ~isfield(vol, 'tra') &  isempty(elc)
  warning('computing potential on all model boundaries');
elseif ~isfield(vol, 'tra') & ~isempty(elc)
  error('there is no electrode transfer matrix present');
end

% concatenate the vertices of all compartment boundaries in a single Nx3 matrix
pnt = [];
for i=1:ncmp
  pnt = [pnt; vol.bnd(i).pnt];
end

% determine the conductivity of the source compartment
if isfield(vol, 'source')
  % use the specified source compartment
  cond = vol.cond(vol.source);
elseif ncmp==1
  % there is only a single compartment which contains the sources
  cond = vol.cond;
else
  % try to locate the innermost compartment and
  % assume that the sources are in the innermost compartment
  warning('BEM model does not specify the source compartment')
  innermost = find_innermost_boundary(vol.bnd);
  warning(sprintf('assuming that compartment %d contains the sources', innermost));
  vol.source = innermost;
  cond = vol.cond(vol.source);
end

% compute infinite medium potential on all vertices
if isfield(vol, 'sigmas')
  % assume "van Oosterom"-type input with no isolated source approach
  lf = [];
  for i=1:length(vol.bnd)
    lf = [lf; inf_medium_leadfield(pos, vol.bnd(i).pnt, mean(vol.sigmas(i,:)))];
  end
else
  % assume input from ASA with isolated potential approach 
  lf = inf_medium_leadfield(pos, pnt, cond);
end

% compute bounded medium potential on all vertices
lf = vol.mat * lf;

% optinally do linear interpolation from vertices to electrodes
if isfield(vol, 'tra')
  lf = vol.tra * lf;	% potential on electrodes
end

