function [vol, sens, cfg] = prepare_vol_sens(cfg, data);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that helps to prepare the electrodes/gradiometers and the volume
% this is used in sourceanalysis and dipolefitting
%
% This function will get the gradiometer/electrode definition from 
%   cfg.channel
%   cfg.gradfile
%   cfg.elecfile
%   cfg.elec
%   cfg.grad
%   data.grad
%   data.elec
% and the volume conductor definition from 
%   cfg.hdmfile
%   cfg.vol
%   data.vol
% Subsequently it will remove the gradiometers/electrodes that are not
% present in the data. Finally it with attach the gradiometers to a
% multi-sphere head model (if supplied) or attach the electrodes to
% a BEM head model.
% 
% This function will return the electrodes/gradiometers in an order that is
% consistent with the original order in the input electrode/gradiometer file
% or definition. This is also reflected in sens.label.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: prepare_vol_sens.m,v $
% Revision 1.11  2005/03/03 10:52:37  roboos
% changed the handling of the channel selection. The input cfg now specifies
% the channels that should be kept in the sensor array. If no selection is
% specified in the input cfg (e.g. for dipolesimulation), the default is 'all'.
%
% Revision 1.10  2005/01/26 08:05:23  roboos
% added empty data if second input argument not given
%
% Revision 1.9  2005/01/17 14:52:50  roboos
% changed to use read_fcdc_elec
%
% Revision 1.8  2004/12/08 18:00:13  roboos
% implemented consistent method of selecting a subset of channels for
% forward and inverse computations using cfg.channel and updated the
% ducumentation
%
% Revision 1.7  2004/09/06 08:45:38  roboos
% moved reading of neuromag BEM bondary from find_inside_vol into prepare_vol_sens
%
% Revision 1.6  2004/09/03 09:15:28  roboos
% added channelselection to the volume for neuromag
%
% Revision 1.5  2004/09/03 06:39:09  roboos
% fixed bug in non-functional neuromag section, cfg->vol
%
% Revision 1.4  2004/09/01 17:30:09  roboos
% added some explanation to the chansel for neuromag
%
% Revision 1.3  2004/08/31 13:55:22  roboos
% added initialization of neuromag megmodel
%
% Revision 1.2  2004/08/06 08:54:53  roboos
% fixed bug for gradiometer info coming from matlab file
%
% Revision 1.1  2004/06/28 08:59:38  roboos
% moved files from fieldtrip to fieldtrip/private
%
% Revision 1.2  2004/05/08 21:06:25  roberto
% added support for reading gradiometer from matlab file
%
% Revision 1.1  2004/04/08 15:51:20  roberto
% initial submissiion into cvs
%

% set the defaults
if ~isfield(cfg, 'channel'), cfg.channel = 'all'; end

if nargin<2
  data = [];
elseif isfield(data, 'label')
  % In the subsequent code, the matching channels in the sensor array and
  % in the configuration will be selected. To ensure that these channels
  % are also present in the data, update the configuration to match the data.
  cfg.channel = channelselection(cfg.channel, data.label);
end

% get the volume conduction model
if isfield(cfg, 'hdmfile')
  fprintf('reading headmodel from file %s\n', cfg.hdmfile);
  switch(filetype(cfg.hdmfile))
    case 'ctf_hdm'
      vol = read_ctf_hdm(cfg.hdmfile);
    case 'asa_vol'
      vol = read_asa_vol(cfg.hdmfile);
    case 'matlab'
      vol = getfield(load(cfg.hdmfile), 'vol');
    case 'neuromag_fif'
      % do not read the volume into Matlab, but use external Neuromag toolbox
      vol.type     = 'neuromag';
      vol.filename = cfg.hdmfile;
      vol.chansel  = [];  % this is defined later based on the channels present in the data
      % initialize the Neuromag toolbox, this requires a gradfile and hdmfile
      fprintf('using Neuromag volume conductor from %s\n', cfg.hdmfile);
      fprintf('using Neuromag gradiometer definition from %s\n', cfg.gradfile);
      megmodel('head', cfg.gradfile, cfg.hdmfile);
      % read the triangulated boundary from the neuromag BEM model
      [vol.bnd.pnt, vol.bnd.tri, normals] = loadtri(vol.filename);
      vol.bnd.pnt = vol.bnd.pnt*100;  %convert to cm
    otherwise
      error('unknown fileformat for volume conductor model');
  end
elseif isfield(cfg, 'vol')
  fprintf('using headmodel specified in the configuration\n');
  vol = cfg.vol;
elseif isfield(data, 'vol') 
  fprintf('using headmodel specified in the data\n');
  vol = data.vol;
else
  error('no headmodel specified');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get the gradiometer positions and orientations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(cfg, 'gradfile') & filetype(cfg.gradfile, 'ctf_res4')
  fprintf('reading gradiometers from file %s\n', cfg.gradfile);
  hdr = read_ctf_res4(cfg.gradfile);
  grad = hdr.grad;
elseif isfield(cfg, 'gradfile') & filetype(cfg.gradfile, 'matlab')
  fprintf('reading gradiometers from file %s\n', cfg.gradfile);
  % read it from a Matlab file
  tmp = load(cfg.gradfile);
  if isfield(tmp, 'grad')
    grad = getfield(tmp, 'grad');
  else
    error('no gradiometers found in Matlab file');
  end
elseif isfield(cfg, 'grad') 
  fprintf('using gradiometers specified in the configuration\n');
  grad = cfg.grad;
elseif isfield(data, 'grad') 
  fprintf('using gradiometers specified in the data\n');
  grad = data.grad;
end

% do postprocessing of the gradiometers
if exist('grad', 'var')
  % update the channelselection with those present in the gradiometer array
  cfg.channel = channelselection(cfg.channel, grad.label);
  % test whether it is a magnetometer or a first-order CTF gradiometer 
  if length(grad.label)==size(grad.pnt,1)
    sel = match_str(grad.label, cfg.channel);
    fprintf('selecting %d magnetometers\n', length(sel));
    grad.label = grad.label(sel);
    grad.pnt   = grad.pnt(sel,:);
    grad.ori   = grad.ori(sel,:);
    if isfield(grad, 'tra')
      grad.tra = sparse(grad.tra(sel, sel));
    end
  elseif length(grad.label)==size(grad.pnt,1)/2
    sel = match_str(grad.label, cfg.channel);
    fprintf('selecting %d first-order gradiometers\n', length(sel));
    % extend the selection to both bottom and top coil
    grad.label = grad.label(sel);
    sel2 = [sel; sel+size(grad.pnt,1)/2];
    grad.pnt   = grad.pnt(sel2,:);
    grad.ori   = grad.ori(sel2,:);
    if isfield(grad, 'tra')
      grad.tra = sparse(grad.tra(sel, sel2));
    end
  else
    sel = match_str(grad.label, cfg.channel);
    warning('higher order synthetic gradiometer configuration(?)');
    grad.label = grad.label(sel);
    % In this case it is not possible to reduce the number of coils, i.e.
    % to make a selection out of pnt and ori. Therefore keep them all and
    % only make a selection out of the linear transformation matrix tra.
    if isfield(grad, 'tra')
      grad.tra = sparse(grad.tra(sel, :));
    else
      error('no transfer matrix present in the gradiometer definition, cannot make channel selection');
    end
  end
  % If the volume conduction model consists of multiple spheres then we
  % have to match the channels in the gradiometer array and the volume
  % conduction model.
  if isfield(vol, 'label') & length(vol.label)>1
    % get the local spheres for the MEG channels, this will be ordered
    % according to the ordering of the gradiometer channels
    [selgrad, selvol] = match_str(grad.label, vol.label);
    vol.label = vol.label(selvol);
    vol.r     = vol.r(selvol);
    vol.o     = vol.o(selvol,:);
    if length(grad.label)==size(grad.pnt,1)/2
      % copy the local spheres for the 2nd coil of each gradiometer
      vol.r     = [vol.r; vol.r];
      vol.o     = [vol.o; vol.o];
    end
  end  
  % if the forward model is computed using the external Neuromag toolbox,
  % we have to add a selection of the channels so that the channels
  % in the forward model correspond with those in the data.
  if isfield(vol, 'type') & strcmp(vol.type, 'neuromag')
    vol.chansel = match_str(grad.label, cfg.channel);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get the electrode positions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(cfg, 'elecfile') 
  fprintf('reading electrodes from file %s\n', cfg.elecfile);
  elec = read_fcdc_elec(cfg.elecfile);
elseif isfield(cfg, 'elec') 
  fprintf('using electrodes specified in the configuration\n');
  elec = cfg.elec;
elseif isfield(data, 'elec') 
  fprintf('using electrodes specified in the data\n');
  elec = data.elec;
end

% do postprocessing of electrodes
if exist('elec', 'var')
  % update the channelselection with those present in the gradiometer array
  cfg.channel = channelselection(cfg.channel, elec.label);
  % select only electrodes present in the data
  sel        = match_str(elec.label, cfg.channel);
  elec.pnt   = elec.pnt(sel,:);
  elec.label = elec.label(sel);
  % create a 2D projection and triangulation
  elec.prj   = elproj(elec.pnt);
  elec.tri   = delaunay(elec.prj(:,1), elec.prj(:,2));
end

% do postprocessing of volume and electrodes in case of BEM model
if exist('elec', 'var') & isfield(vol, 'bnd') & ~isfield(vol, 'tra')
  % determine boundary corresponding with skin and brain
  skin   = find_outermost_boundary(vol.bnd);
  brain  = find_innermost_boundary(vol.bnd);
  vol.source = brain;       % remember this for faster forward computation
  fprintf('projecting electrodes on skin surface (%d)\n', skin);
  % compute linear interpolation from triangle vertices towards electrodes
  el   = project_elec(elec.pnt, vol.bnd(skin).pnt, vol.bnd(skin).tri);
  tra  = transfer_elec(vol.bnd(skin).pnt, vol.bnd(skin).tri, el);
  % ensure that the model potential is average referenced
  tra = avgref(tra);
  % construct the transfer from all vertices (also brain/skull) towards electrodes
  vol.tra = [];
  for i=1:length(vol.bnd)
    if i==skin
      vol.tra = [vol.tra, tra];
    else
      vol.tra = [vol.tra, zeros(size(el,1), size(vol.bnd(i).pnt,1))];
    end
  end
  % convert to sparse matrix to speed multiplications up
  vol.tra = sparse(vol.tra);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% restructure the output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if exist('elec', 'var')
  sens = elec;
  sens.type = 'electrodes';
elseif exist('grad', 'var')
  sens = grad;
  sens.type = 'gradiometers';
else
  error('cannot find electrodes or gradiometers'); 
end

% update the configuration, so that the calling function exactly knows wat was selected
cfg.channel = sens.label;
