function [dat, label, time, cfg] = preproc(dat, label, fsample, cfg, offset, begpadding, endpadding);

% PREPROC applies various preprocessing steps on a piece of EEG/MEG data
% that already has been read from a data file. 
%
% This function can serve as a subfunction for all FieldTrip modules that
% want to preprocess the data, such as PREPROCESSING, ARTIFACT_XXX,
% TIMELOCKANALYSIS, etc. It ensures consistent handling of both MEG and EEG
% data and consistency in the use of all preprocessing configuration
% options.
%
% Use as
%   [dat, label, time, cfg] = preproc(dat, label, fsample, cfg, offset, begpadding, endpadding)
%
% The required input arguments are
%   dat         Nchan x Ntime data matrix
%   label       Nchan x 1 cell-array with channel labels
%   cfg         configuration structure, see below
%   fsample     sampling frequency
% and the optional input arguments are
%   offset      of the first datasample (see below)
%   begpadding  number of samples used for padding (see below)
%   endpadding  number of samples used for padding (see below)
%
% The output is
%   dat         Nchan x Ntime data matrix
%   label       Nchan x 1 cell-array with channel labels
%   time        Ntime x 1 vector with the latency in seconds
%   cfg         configuration structure, optionally with extra defaults set
%
% Note that the number of input channels and the number of output channels
% can be different, for example when the user specifies that he/she wants
% to add the implicit EEG reference channel to the data matrix.
%
% The offset field specifies the difference in the latency of the beginning
% of the data relative to the occurence of the trigger (expressed in
% samples). An offset of 0 means that the first sample of the trial
% corresponds with the trigger. A positive offset indicates that the first
% sample is later than the triger, a negative offset indicates that the
% trial begins before the trigger. The offset should be specified EXCLUDING
% the filter padding at the begin of the data. You can leave it empty,
% which implies that the first sample of the data corresponds with latency 0.
%
% The filtering of the data can introduce artifacts at the edges, hence it
% is better to pad the data with some extra signal at the begin and end.
% After filtering, this padding is removed and the other preprocessing
% steps are applied to the remainder of the data. The input fields
% begpadding and endpadding should be specified in samples. You can also
% leave them empty, which implies that the data is not padding.
%
% The configuration can contain
%   cfg.lpfilter      = 'no' or 'yes'  lowpass filter
%   cfg.hpfilter      = 'no' or 'yes'  highpass filter
%   cfg.bpfilter      = 'no' or 'yes'  bandpass filter
%   cfg.lnfilter      = 'no' or 'yes'  line noise removal using notch filter
%   cfg.dftfilter     = 'no' or 'yes'  line noise removal using discrete fourier transform
%   cfg.medianfilter  = 'no' or 'yes'  jump preserving median filter
%   cfg.lpfreq        = lowpass  frequency in Hz
%   cfg.hpfreq        = highpass frequency in Hz
%   cfg.bpfreq        = bandpass frequency range, specified as [low high] in Hz
%   cfg.lnfreq        = line noise frequency in Hz, default 50Hz
%   cfg.lpfiltord     = lowpass  filter order
%   cfg.hpfiltord     = highpass filter order
%   cfg.bpfiltord     = bandpass filter order
%   cfg.lnfiltord     = line noise notch filter order
%   cfg.medianfiltord = length of median filter
%   cfg.detrend       = 'no' or 'yes'
%   cfg.blc           = 'no' or 'yes'
%   cfg.blcwindow     = [begin end] in seconds, the default is the complete trial
%   cfg.hilbert       = 'no' or 'yes'
%   cfg.rectify       = 'no' or 'yes'
%   cfg.implicitref   = 'label' or empty, add the implicit EEG reference as zeros
%   cfg.reref         = 'no' or 'yes'
%   cfg.refchannel    = cell-array with new reference channel(s)
%
% See also READ_FCDC_DATA, READ_FCDC_HEADER

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: preproc.m,v $
% Revision 1.2  2005/01/21 09:53:11  roboos
% implemented median filter in preproc, updated help
%
% Revision 1.1  2004/12/09 17:22:28  roboos
% initial version, replicates all preprocessing steps from the large preprocessing function such as filtering, detrending and re-referencing.
%

% set the defaults, this is copy-and-paste from PREPROCESSING
if ~isfield(cfg, 'detrend'),      cfg.detrend = 'no';           end
if ~isfield(cfg, 'blc'),          cfg.blc = 'no';               end
if ~isfield(cfg, 'blcwindow'),    cfg.blcwindow = 'all';	      end
if ~isfield(cfg, 'lnfilter'),     cfg.lnfilter = 'no';          end
if ~isfield(cfg, 'dftfilter'),    cfg.dftfilter = 'no';         end
if ~isfield(cfg, 'lpfilter'),     cfg.lpfilter = 'no';          end
if ~isfield(cfg, 'hpfilter'),     cfg.hpfilter = 'no';          end
if ~isfield(cfg, 'bpfilter'),     cfg.bpfilter = 'no';          end
if ~isfield(cfg, 'lpfiltord'),    cfg.lpfiltord = 6;            end
if ~isfield(cfg, 'hpfiltord'),    cfg.hpfiltord = 6;            end
if ~isfield(cfg, 'bpfiltord'),    cfg.bpfiltord = 4;            end
if ~isfield(cfg, 'lnfiltord'),    cfg.lnfiltord = 4;            end
if ~isfield(cfg, 'lpfilttype'),   cfg.lpfilttype = 'but';       end
if ~isfield(cfg, 'hpfilttype'),   cfg.hpfilttype = 'but';       end
if ~isfield(cfg, 'bpfilttype'),   cfg.bpfilttype = 'but';       end
if ~isfield(cfg, 'medianfilter'), cfg.medianfilter  = 'no';     end
if ~isfield(cfg, 'medianfiltord'),cfg.medianfiltord = 9;        end
if ~isfield(cfg, 'lnfreq'),       cfg.lnfreq = 50;              end
if ~isfield(cfg, 'hilbert'),      cfg.hilbert = 'no';           end
if ~isfield(cfg, 'rectify'),      cfg.rectify = 'no';           end
if ~isfield(cfg, 'reref'),        cfg.reref = 'no';             end
if ~isfield(cfg, 'refchannel'),   cfg.refchannel = {};          end
if ~isfield(cfg, 'implicitref'),  cfg.implicitref = [];         end

% test whether the Matlab signal processing toolbox is available
hassignalproc = exist('medfilt1');
if strcmp(cfg.medianfilter, 'yes') & ~hassignalproc
  error('median filtering requires the Matlab signal processing toolbox');
end

if nargin<5 | isempty(offset)
  offset = 0;
end
if nargin<6 | isempty(begpadding)
  begpadding = 0;
end
if nargin<7 | isempty(endpadding)
  endpadding = 0;
end

if strcmp(cfg.medianfilter, 'yes'), dat = medfilt1(dat, cfg.medianfiltord, [], 2); end
if strcmp(cfg.lpfilter, 'yes'),  dat = lowpassfilter (dat, fsample, cfg.lpfreq, cfg.lpfiltord, cfg.lpfilttype); end
if strcmp(cfg.hpfilter, 'yes'),  dat = highpassfilter(dat, fsample, cfg.hpfreq, cfg.hpfiltord, cfg.hpfilttype); end
if strcmp(cfg.bpfilter, 'yes'),  dat = bandpassfilter(dat, fsample, cfg.bpfreq, cfg.bpfiltord, cfg.bpfilttype); end
if strcmp(cfg.lnfilter, 'yes'),  dat = notchfilter   (dat, fsample, cfg.lnfreq, cfg.lnfiltord); end
if strcmp(cfg.dftfilter, 'yes'),
  dat = dftfilter(dat, fsample, cfg.lnfreq);
  dat = dftfilter(dat, fsample, cfg.lnfreq*2);     % also filter out 100Hz
  dat = dftfilter(dat, fsample, cfg.lnfreq*3);     % also filter out 150Hz
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remove the filter padding and do the preprocessing on the remaining trial data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dat = dat(:, (1+begpadding):(end-endpadding));
% determine the timeaxis for this trial
nsamples = size(dat,2);
time     = (offset + (0:(nsamples-1)))/fsample;
if ~isempty(cfg.implicitref)
  label = {label{:} cfg.implicitref};
  dat(end+1,:) = 0;
end
if strcmp(cfg.reref, 'yes'),
  cfg.refchannel = channelselection(cfg.refchannel, label);
  refindx = match_str(label, cfg.refchannel);
  if isempty(refindx)
    error('reference channel was not found')
  end
  dat = avgref(dat, refindx);
end
if strcmp(cfg.detrend, 'yes')
  dat = detrend(dat')';
end
if strcmp(cfg.blc, 'yes')
  if isstr(cfg.blcwindow) & strcmp(cfg.blcwindow, 'all')
    dat = blc(dat);
  else
    % determine the begin and endsample of the baseline period and baseline correct for it
    blcbegsample = nearest(time, cfg.blcwindow(1));
    blcendsample = nearest(time, cfg.blcwindow(2));
    dat = blc(dat, blcbegsample, blcendsample);
  end
end
if strcmp(cfg.hilbert, 'yes'),
  dat = abs(hilbert(dat'))';
end
if strcmp(cfg.rectify, 'yes'),
  dat = abs(dat);
end
