function p = randstatprob(rand, obs, tail, correctm);

% RANDSTATPROB computes the non-parametric probability of the observed
% value under the assumption that the random observations are equally
% probable under the null hypothesis.
%
% Use as
%   p = randstatprob(rand, obs, tail, correctm)
% where
%   rand     = Nvox x Nrnd
%   obs      = Nvox x 1, or Nvox x Nobs (for multiple observations)
%   tail     =  0 for two-sided test
%   tail     =  1 for one-sided test with obs>=rand
%   tail     = -1 for one-sided test with obs<=rand
%   correctm = 0 or 1, apply multiple comparison correction
%
% Each row of the input data contains all the (real or randomized)
% observations in one voxel. Multiple comparison can be performed by 
% creating a reference distribution based on the minimum or maximum 
% of all voxels for each randomization.

% FIXME 
% the explanation in the help should be made statistically more sound

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: randstatprob.m,v $
% Revision 1.2  2004/11/15 11:17:55  roboos
% added multiple comparison correction
%
% Revision 1.1  2004/11/11 17:24:19  roboos
% new implementation
%

if nargin<3
  tail = 0;
end

if nargin<4
  correctm = 0;
end

Nvox = size(rand,1);
Nrnd = size(rand,2);
Nobs = size(obs,2);
if size(obs,1)~=Nvox
  error('dimensions of input arguments does not match');
end

p = zeros(size(obs));

if correctm
  % the distribution of the min/max statistic is the same for each voxel
  mindist = min(rand, [], 1);
  maxdist = max(rand, [], 1);
end

for i=1:Nobs
  if correctm
    switch tail
      case 0
        % the observation can be either in the tail with the smallest or the tail with the largest values
        plo = sum(repmat(mindist,Nvox,1) <= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
        phi = sum(repmat(maxdist,Nvox,1) >= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
        p(:,i) = min(plo,phi);
    
      case 1
        p(:,i) = sum(repmat(maxdist,Nvox,1) >= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
    
      case -1
        p(:,i) = sum(repmat(mindist,Nvox,1) <= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
    
      otherwise
        error('incorrect specification of tail');
    end
  elseif ~correctm
    switch tail
      case 0
        plo = sum(rand <= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
        phi = sum(rand >= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
        p(:,i) = min(plo,phi);
    
      case 1
        p(:,i) = sum(rand >= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
    
      case -1
        p(:,i) = sum(rand <= repmat(obs(:,i),1,Nrnd), 2)./Nrnd;
    
      otherwise
        error('incorrect specification of tail');
    end
  end
end
