function sourceplot(cfg, interp)

% SOURCEPLOT displays three orthogonal slices of anatomical and/or functional
% volume data. If an anatomical volume is present in the input, it will
% always be plotted. The source parameter that is specified in the
% configuration will be plotted as an overlay on the anatomy, where the
% mask parameter can be used to determine the opacity.
%
% Use as
%   sourceplot(cfg, source)  or
%   sourceplot(cfg, interp)  or
%   sourceplot(cfg, stat)    or
%   sourceplot(cfg, mri)
% where
%   source is the output of SOURCEANALYSIS,
%   interp is the output of SOURCEINTERPOLATE,
%   stat   is the output of SOURCESTATISTICS,
%   mri    is the output of READ_FCDC_MRI or an MRI filename
% and cfg is a structure with any of the following fields
%   cfg.funparameter  = string
%   cfg.maskparameter = string
%   cfg.location      = 'min', 'max', 'interactive' or [x y z]
%
%
% See also SOURCEANALYSIS, SOURCESTATISTICS, SOURCEINTERPOLATE

% Copyright (C) 2003, Robert Oostenveld
%
% $Log: sourceplot.m,v $
% Revision 1.13  2005/03/03 10:38:26  roboos
% changed axes labels from x/y/z into i/j/k, since axes correspond with voxel indices
%
% Revision 1.12  2005/02/28 17:58:59  roboos
% fixed bug in selection of empty funparameter
%
% Revision 1.11  2005/02/21 12:29:08  roboos
% removed incorrect inv() from interp.transform when interactive plotting
%
% Revision 1.10  2005/02/11 17:59:01  roboos
% fixed bug in computation of headcoordinates (removed inv)
%
% Revision 1.9  2005/02/11 14:49:11  roboos
% moved clipping and scaling from subfunction to mainfunction
% implemented cfg.downsample
%
% Revision 1.8  2005/02/11 13:28:53  roboos
% completely new implementation, basically based upon misc/volplot and on Nienke's singleplot
%
% Revision 1.7  2004/08/26 10:48:25  roboos
% modified to make it consistent again with the new volume (source+mri) structure
% removed dependency on warp3d, implemented homogenous transform directly

% sometimes it is desirable to specify the cfg as a cell array
if iscell(cfg)
  var = cfg(1:2:length(cfg)); % select the odd elements
  val = cfg(2:2:length(cfg)); % select the even elements
  % convert the configuration to a structure
  cfg = cell2struct(val, var, 2);
end

% set the defaults
if ~isfield(cfg, 'location');        cfg.location = 'interactive';   end
if ~isfield(cfg, 'funparameter');    cfg.funparameter = [];          end
if ~isfield(cfg, 'maskparameter');   cfg.maskparameter = [];         end
if ~isfield(cfg, 'colormap');        cfg.colormap = 'jet';           end
if ~isfield(cfg, 'downsample');      cfg.downsample = 1;             end

if isstr(interp)
  % read the anatomical MRI data from file
  filename = interp;
  fprintf('reading MRI from file\n');
  interp = read_fcdc_mri(filename);
end

% collect the volumes that will be plotted
ana = [];
fun = [];
msk = [];

% take care that xgrid=1:Nx etc., and that the transformation matrix holds the coordinates
if ~isfield(interp, 'transform')
  interp.transform = eye(4);
end
[interp.xgrid, interp.ygrid, interp.zgrid, interp.transform] = transformgrid(interp.xgrid, interp.ygrid, interp.zgrid, interp.transform);

% downsample the volume if requested
if cfg.downsample~=1
  tmpcfg = [];
  tmpcfg.downsample = cfg.downsample;
  interp = downsamplevolume(tmpcfg, interp);
end

if isfield(interp, 'anatomy')
  hasana = 1;
  mri8  = isa(interp.anatomy, 'uint8');
  mri16 = isa(interp.anatomy, 'uint16');
  % convert integers to single precision float if neccessary
  if mri8 | mri16
    fprintf('converting anatomy to double\n');
    ana = double(interp.anatomy);
  else
    ana = interp.anatomy;
  end
else
  hasana = 0;
  fprintf('no anatomical volume present\n');
  ana = ones(interp.dim);
end

if isfield(cfg, 'funparameter') & ~isempty(cfg.funparameter) & ~isempty(strmatch(cfg.funparameter, {'pow', 'coh', 'nai', 'noise'}))
  % assume that these parameters can be found under avg
  cfg.funparameter = ['avg.' cfg.funparameter];
end

if ~isempty(cfg.funparameter) & issubfield(interp, cfg.funparameter)
  hasfun = 1;
  fun = getsubfield(interp, cfg.funparameter);
else
  hasfun = 0;
  cfg.funparameter = [];
  fun = zeros(interp.dim);
end

if ~isempty(cfg.maskparameter) & issubfield(interp, cfg.maskparameter)
  hasmsk = 1;
  msk = getsubfield(interp, cfg.maskparameter);
else
  hasmsk = 0;
  cfg.maskparameter = [];
  if isempty(cfg.funparameter)
    msk = zeros(interp.dim);
  else
    msk = 0.7 * ones(interp.dim);
  end
end

% ensure that they are all 3D volumes
ana = reshape(ana, interp.dim);
fun = reshape(fun, interp.dim);
msk = reshape(msk, interp.dim);

% ensure that the functional data is real
if ~isreal(fun)
  fprintf('taking absolute value of complex data\n');
  fun = abs(fun);
end

anasc = [];
funsc = [];
msksc = [];

% automatically determine the interesting range of values that should be plotted
if isempty(anasc) & hasana
  anasc(1) = 0;
  anasc(2) = max(ana(:));
else
  anasc = [0 1];
end
if isempty(funsc) & hasfun
  funsc(1) = min(fun(:));
  funsc(2) = max(fun(:));
else
  funsc = [0 1];
end
if isempty(msksc) & hasmsk
  msksc(1) = 0;
  msksc(2) = 1;
else
  msksc = [0 1];
end

x = interp.xgrid;
y = interp.ygrid;
z = interp.zgrid;
dim = interp.dim;

if ~isstr(cfg.location)
  % select the headcoordinates location in voxel coordinates
  sel = inv(interp.transform) * [cfg.location(:); 1];
  sel = round(sel(1:3));
else
  sel = cfg.location;
end

% determine the initial intersection of the cursor
if isstr(sel) & strcmp(sel, 'min')
  if isempty(cfg.funparameter)
    error('no functional parameter specified');
  end
  [minval, minindx] = min(fun(:));
  [xi, yi, zi] = ind2sub(interp.dim, minindx);
elseif isstr(sel) & strcmp(sel, 'max')
  if isempty(cfg.funparameter)
    error('no functional parameter specified');
  end
  [maxval, maxindx] = max(fun(:));
  [xi, yi, zi] = ind2sub(interp.dim, maxindx);
elseif isstr(sel) & strcmp(sel, 'center')
  xi = round(length(x)/2);
  yi = round(length(y)/2);
  zi = round(length(z)/2);
elseif isstr(sel) & strcmp(sel, 'interactive')
  % start at the center
  xi = round(length(x)/2);
  yi = round(length(y)/2);
  zi = round(length(z)/2);
elseif ~isstr(sel)
  xi = nearest(x, sel(1));
  yi = nearest(y, sel(2));
  zi = nearest(z, sel(3));
end

% define the colormap for the functional data
funcolormap = colormap(cfg.colormap);

nas = [];
lpa = [];
rpa = [];

if ~strcmp(cfg.location, 'interactive')
  % plot only once
  xi = round(xi); xi = max(xi, 1); xi = min(xi, dim(1));
  yi = round(yi); yi = max(yi, 1); yi = min(yi, dim(2));
  zi = round(zi); zi = max(zi, 1); zi = min(zi, dim(3));

  ijk = [xi yi zi 1]';
  xyz = interp.transform * ijk;
  val = fun(xi, yi, zi);
  fprintf('voxel %d, indices [%d %d %d], location [%.1f %.1f %.1f] mm, value %f\n', sub2ind(interp.dim, xi, yi, zi), ijk(1:3), xyz(1:3), val);

  subplot(2,2,1); singleplot(ana, fun, msk, anasc, funsc, msksc, [xi yi zi], 2, funcolormap); xlabel('i'); ylabel('k'); crosshair([xi zi]);
  subplot(2,2,2); singleplot(ana, fun, msk, anasc, funsc, msksc, [xi yi zi], 1, funcolormap); xlabel('j'); ylabel('k'); crosshair([yi zi]);
  subplot(2,2,3); singleplot(ana, fun, msk, anasc, funsc, msksc, [xi yi zi], 3, funcolormap); xlabel('i'); ylabel('j'); crosshair([xi yi]);
  drawnow;

else
  % keep on plotting until the user presses quit
  while(1)

    xi = round(xi); xi = max(xi, 1); xi = min(xi, dim(1));
    yi = round(yi); yi = max(yi, 1); yi = min(yi, dim(2));
    zi = round(zi); zi = max(zi, 1); zi = min(zi, dim(3));

    fprintf('\n');
    fprintf('click with mouse button to reposition the cursor\n');
    fprintf('press q on keyboard to quit interactive mode\n');
    if ~isempty(nas), fprintf('nas = [%f %f %f]\n', nas); end
    if ~isempty(lpa), fprintf('lpa = [%f %f %f]\n', lpa); end
    if ~isempty(rpa), fprintf('rpa = [%f %f %f]\n', rpa); end

    ijk = [xi yi zi 1]';
    xyz = interp.transform * ijk;
    val = fun(xi, yi, zi);
    fprintf('voxel %d, indices [%d %d %d], location [%.1f %.1f %.1f] mm, value %f\n', sub2ind(interp.dim, xi, yi, zi), ijk(1:3), xyz(1:3), val);

    subplot(2,2,1); singleplot(ana, fun, msk, anasc, funsc, msksc, [xi yi zi], 2, funcolormap); xlabel('i'); ylabel('k'); crosshair([xi zi]);
    subplot(2,2,2); singleplot(ana, fun, msk, anasc, funsc, msksc, [xi yi zi], 1, funcolormap); xlabel('j'); ylabel('k'); crosshair([yi zi]);
    subplot(2,2,3); singleplot(ana, fun, msk, anasc, funsc, msksc, [xi yi zi], 3, funcolormap); xlabel('i'); ylabel('j'); crosshair([xi yi]);
    drawnow;

    try, [d1, d2, key] = ginput(1); catch, key='q'; end
    if key=='q'
      break;
    elseif key=='l'
      lpa = [xi yi zi];
    elseif key=='r'
      rpa = [xi yi zi];
    elseif key=='n'
      nas = [xi yi zi];
    else
      % update the view to a new position
      l1 = get(get(gca, 'xlabel'), 'string');
      l2 = get(get(gca, 'ylabel'), 'string');
      switch l1,
        case 'i'
          xi = d1;
        case 'j'
          yi = d1;
        case 'k'
          zi = d1;
      end
      switch l2,
        case 'i'
          xi = d2;
        case 'j'
          yi = d2;
        case 'k'
          zi = d2;
      end
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SINGLEPLOT makes an overlay of 3D anatomical, functional and probability
% volumes. You can precisely specify the scaling of each type of data,
% and you can specify at which location and along which dimension the
% slice will be shown.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function singleplot(ana, fun, msk, anasc, funsc, msksc, indx, dimension ,funcolormap);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SINGLEPLOT makes an overlay of 3D anatomical, functional and probability
% volumes. The three volumes must be scaled between 0 and 1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function singleplot(ana, fun, msk, anasc, funsc, msksc, indx, dimension ,funcolormap);

% select the indices of the intersection
xi = indx(1);
yi = indx(2);
zi = indx(3);

% select the slice to plot
if dimension==1
  yi = 1:size(ana,2);
  zi = 1:size(ana,3);
elseif dimension==2
  xi = 1:size(ana,1);
  zi = 1:size(ana,3);
elseif dimension==3
  xi = 1:size(ana,1);
  yi = 1:size(ana,2);
end

% cut out the slice of interest
ana = squeeze(ana(xi,yi,zi));
fun = squeeze(fun(xi,yi,zi));
msk = squeeze(msk(xi,yi,zi));

% fprintf('scaling and clipping anatomy\n');
ana(find(ana(:)<anasc(1))) = anasc(1);      % clip to minimal interesting values
ana(find(ana(:)>anasc(2))) = anasc(2);      % clip to maximal interesting values
ana = (ana-anasc(1))./(anasc(2)-anasc(1));  % scale interesting range to 0-1
% fprintf('scaling and clipping functional\n');
fun(find(fun(:)<funsc(1))) = funsc(1);      % clip to minimal values
fun(find(fun(:)>funsc(2))) = funsc(2);      % clip to maximal values
fun = (fun-funsc(1))./(funsc(2)-funsc(1));  % scale interesting range to 0-1
% fprintf('scaling and clipping mask\n');
msk(find(msk(:)<msksc(1))) = msksc(1);      % clip to minimal interesting values
msk(find(msk(:)>msksc(2))) = msksc(2);      % clip to maximal interesting values
msk = (msk-msksc(1))./(msksc(2)-msksc(1));  % scale interesting range to 0-1

% this is needed for displaying the matrix using the Matlab image() function
ana = ana';
fun = fun';
msk = msk';

dim = size(ana);

% convert anatomy into RGB values
ana = cat(3, ana, ana, ana);

% convert functional into RGB values
fun = floor((size(funcolormap,1)-1) * fun)+1;	% scale interesting range to 1-64
fun(find(isnan(fun(:)))) = 1;
r = zeros(dim);
g = zeros(dim);
b = zeros(dim);
r(:) = funcolormap(fun(:), 1);			% find the matching color (Rgb)
g(:) = funcolormap(fun(:), 2);			% find the matching color (rGb)
b(:) = funcolormap(fun(:), 3);			% find the matching color (rgB)
fun = cat(3, r, g, b);

ha = image(ana);
hold on
hf = image(fun);
set(hf, 'AlphaData', msk);          % apply the opacity mask to the functional data
axis equal
axis tight
axis xy

