function [stat] = timelockstatistics(cfg, varargin)

% TIMELOCKSTATISTICS computes statistics for ERF/ERPs
% in different conditions
%
% Use as
%   [stat] = timelockstatistics(cfg, cond1, cond2, ...)
%
% where
%   cond1..N are the grand-averages for the different conditions
% as obtained from TIMELOCKGRANDAVERAGE
%
% and cfg is a configuration structure with
%  cfg.channel        = cell-array with channel labels (can be 'all')
%  cfg.latency        = [begin end]
%  cfg.method         = string with statistical method, can be
%                       'ttest_samples_vs_const' 
%                       'ttest_window_avg_vs_const' 
%                       'ttest_2samples_by_timepoint' 
%
%  cfg.alpha           = critical alpha for rejecting Null-hypothesis
%  cfg.tail
%    The Null hypothesis is: "mean is equal to M".
%     For TAIL=0,  alternative: "mean is not M".
%     For TAIL=1,  alternative: "mean is greater than M"
%     For TAIL=-1, alternative: "mean is less than M"
%     TAIL = 0 by default.
%
% See also TIMELOCKANALYSIS, TIMELOCKGRANDAVERAGE

% Copyright (C) 2004 Jens Schwarzbach
% Universiteit Maastricht/FCDonders Centre
%
% $Log: timelockstatistics.m,v $
% Revision 1.7  2004/10/13 14:11:30  roboos
% changed cfg.previous, now consistent over functions with try-statement
% and will also work if the input already had a cfg.previous
%
% Revision 1.6  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%
% Revision 1.5  2004/06/30 15:05:53  roboos
% converted ascii file from DOS to UNIX format, no functional code changes
%
% Revision 1.4  2004/06/29 15:38:14  roboos
% fixed bug in cfg.latency=all default
% implemented support for comparing conditions within one subject, i.e. without grandaveraging the data
%
% Revision 1.3  2004/06/29 10:33:01  jensch
% added log feature of CVS
%

% set the defaults
if ~isfield(cfg, 'channel'),     cfg.channel = 'all';	end
if ~isfield(cfg, 'latency'),     cfg.latency = 'all';	end
if ~isfield(cfg, 'alpha'),       cfg.alpha = .05;	    end
if ~isfield(cfg, 'tail'),        cfg.tail = 0;	      end

% varargin{1} ... varargin{end} contain the different conditions
Ncond = length(varargin);

% replace latency selection by real timerange
if ischar(cfg.latency) & strcmp(cfg.latency, 'all')
  % this will be replaced by the real minimum and maximum later on
  cfg.latency = [-inf inf];
end

%SELECT TIME WINDOW
idxs = nearest(varargin{1}.time, min(cfg.latency));
idxe = nearest(varargin{1}.time, max(cfg.latency));
ResultsTimeSelectCases = idxs:idxe;
ResultsNTimePoints = length(ResultsTimeSelectCases);
ResultsTime = varargin{1}.time(ResultsTimeSelectCases);
%UPDATE CFG STRUCTURE WITH TIME THAT WAS FINALLY USED
cfg.latency = [ResultsTime(1), ResultsTime(end)];

% determine what kind of data was presented to this function
if isfield(varargin{1}, 'trial') 
  fprintf('comparing conditions within one subject\n');
  % rename varargin{i}.trial to varargin{i}.individual and add nsubjects
  % so that we later only have to support one datatype
  for i=1:Ncond
    varargin{i}.individual = varargin{i}.trial;
    varargin{i}.nsubjects = size(varargin{i}.trial, 1);
    varargin{i} = rmfield(varargin{i}, 'trial');
  end
else
  fprintf('comparing conditions between subjects\n');
  % no conversion needed
end

%REPLACE WITH PROPER CHANNEL SELECTION
%REDUCE DATASET TO INTERSECTION OF DESIRED AND AVAILABLE CHANNELS 
% determine the channels of interest 
cfg.channel = channelselection(cfg.channel, varargin{1}.label);
chansel     = match_str(varargin{1}.label, cfg.channel);

for i=1:Ncond
    %chansel = match_str(varargin{i}.label, cfg.channel);
    varargin{i}.avg = varargin{i}.avg(chansel,:);
    varargin{i}.label = varargin{i}.label(chansel);
    %DOES THIS MAKE SENSE?
    try, varargin{i}.individual = varargin{i}.individual(:, chansel,:); end
end
%WHAT HAPPENS IF THERE IS NO MATCH BETWEEN DESIRED AND AVAILABLE CHANNELS?
%cfg.channel = channelselection(cfg.channel, chansel);
ResultsNChannels = size(cfg.channel, 1);


%PUT AVG DATA IN A 3D MATRIX [condition, channel, time]
%PREALLOCATE MEMORY
SizeOfData = [ResultsNChannels, ResultsNTimePoints];%[size(varargin{1}.dataAVG.avg);
avgmat = zeros(Ncond, ResultsNChannels, ResultsNTimePoints);
varmat = zeros(Ncond, ResultsNChannels, ResultsNTimePoints);

if isfield(varargin{1}, 'individual')
    %PREALLOCATE MATRIX Nsubjects, Nchannels, Ntimepoints
    %use zeros
    indmat = zeros(Ncond, varargin{1}.nsubjects, ResultsNChannels, ResultsNTimePoints);
    %indmat = repmat(0, [Ncond, varargin{1}.nsubjects, SizeOfData]);
end

%FILL MATRIX, MAY BE DONE MORE EFFECTIVELY WITH DEAL COMMAND
for cond = 1:Ncond
    avgmat(cond, :, :) = varargin{cond}.avg(:, ResultsTimeSelectCases);
    %BUG IN AVERAGING ROUTINE, VARIANCE SHOULD BE THE DEVIATION OF THIS
    %SUBJECT FROM GROUP MEAN
    %varmat(cond, :, :) = varargin{cond}.var(:, ResultsTimeSelectCases);
    %INDIVIDUAL DATA NEEDED FOR TTEST AGAINST CONST
    if isfield(varargin{cond}, 'individual')
        %indmat(conditions, subjects, Channels, time)
        indmat(cond, :, :, :) = varargin{cond}.individual(:, :, ResultsTimeSelectCases);
    end
end
stat = [NaN];

if exist('indmat') == 1
    %indmat does not always exist
    WindowAVG = mean(indmat, 4); %AVERAGE ACROSS ALL SAMPLES
else
    WindowAVG = [];
    indmat = [];
end

switch cfg.method
    case 'ttest_samples_vs_const'
        %NCONDITIONS * NCHANNELS * NTIMEPOINTS
        h = zeros(Ncond, ResultsNChannels, ResultsNTimePoints);
        p = zeros(Ncond, ResultsNChannels, ResultsNTimePoints);
        ci = zeros(Ncond, ResultsNChannels, ResultsNTimePoints, 2);
        
        for cond = 1:Ncond
            fprintf(1, 'TTEST condition %3d/%3d\n', cond, Ncond); 
            fprintf(1, 'Channel ')
            for cha = 1:ResultsNChannels
                fprintf(1, '%4d', cha)
                for s = 1:ResultsNTimePoints
                    %[H,SIG] = TTEST(X,M,ALPHA,TAIL) should be part of cfg
                    [h(cond, cha, s), p(cond, cha, s), ci(cond, cha, s, :)] = ttest(indmat(cond, :, cha, s), cfg.constantvalue, cfg.alpha, 0);
                end
                fprintf(1, '\b\b\b\b')
            end
            fprintf(1, '\n')
        end
        
    case 'ttest_window_avg_vs_const'
        h = zeros(Ncond, ResultsNChannels);
        p = zeros(Ncond, ResultsNChannels);
        ci = zeros(Ncond, ResultsNChannels, 2);
        
        for cond = 1:Ncond
            fprintf(1, 'TTEST condition %3d/%3d\n', cond, Ncond); 
            for cha = 1:ResultsNChannels
                %[H,SIG] = TTEST(X,M,ALPHA,TAIL) should be part of cfg
                [h(cond, cha), p(cond, cha), ci(cond, cha, :)] = ttest(WindowAVG(cond, :, cha), 0, cfg.alpha, 0);
            end
        end
        
    case 'ttest_2samples_by_timepoint'
        %NCONDITIONS * NCHANNELS * NTIMEPOINTS
        %NCond must be 2
        h = zeros(ResultsNChannels, ResultsNTimePoints);
        p = zeros(ResultsNChannels, ResultsNTimePoints);
        ci = zeros(ResultsNChannels, ResultsNTimePoints, 2);
        
        fprintf(1, 'Channel ')
        for cha = 1:ResultsNChannels
            fprintf(1, '%4d', cha)
            for s = 1:ResultsNTimePoints
                %[H,SIG] = TTEST(X,M,ALPHA,TAIL) should be part of cfg
                %[h(cha, s), p(cha, s), ci(cha, s, :)] = ttest(indmat(cond, :, cha, s), cfg.constantvalue, .05, 0);
                [h(cha, s), p(cha, s), ci(cha, s, :)] = ttest2(indmat(1, :, cha, s), indmat(2, :, cha, s), cfg.alpha, 0);
                
            end
            fprintf(1, '\b\b\b\b')
        end
        fprintf(1, '\n')
        
    otherwise
        fprintf(1, 'Statistical method %s is not implemented', cfg.method);
        
end

stat.h = h;
stat.p = p;
stat.ci = ci;
stat.time = ResultsTime;
stat.label = varargin{1}.label;
stat.cfg = cfg;
%indmat does not always exist, WindowAVG neither
stat.WindowMean = squeeze(mean(WindowAVG, 2)); %average across subjects and samples
stat.SampleMean = squeeze(mean(indmat, 2)); %average across subjects

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: timelockstatistics.m,v 1.7 2004/10/13 14:11:30 roboos Exp $';

% remember the configuration details from the input data
cfg.previous = [];
for i=1:length(varargin)
  try, cfg.previous{i} = varargin{i}.cfg; end
end

% add the full configuration details to the output 
stat.cfg = cfg;
