function [output] = rotate(R, input);

% ROTATE performs a 3D rotation on the input coordinates
% around the x, y and z-axis. The direction of the rotation 
% is according to the right-hand rule. The rotation is first
% done around the x-, then the y- and finally the z-axis.
% 
% Use as
%   [output] = rotate(R, input)
% where
%   R		[rx, ry, rz] rotations around each of the axes in degrees
%   input	Nx3 matrix with the points before rotation
%   output	Nx3 matrix with the points after rotation
%
% Or as
%   [Tr] = rotate(R)
% where
%   R		[rx, ry, rz] in degrees
%   Tr		corresponding homogenous transformation matrix

% Copyright (C) 2000-2004, Robert Oostenveld
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% $Log: rotate.m,v $
% Revision 1.3  2004/05/19 09:57:07  roberto
% added GPL copyright statement, added CVS log item
%

rotx = eye(3);
roty = eye(3);
rotz = eye(3);

rx = pi*R(1) / 180;
ry = pi*R(2) / 180;
rz = pi*R(3) / 180;

if rx~=0
  % rotation around x-axis
  rotx(2,:) = [   0    cos(rx)  -sin(rx) ];
  rotx(3,:) = [   0    sin(rx)   cos(rx) ];
end

if ry~=0
  % rotation around y-axis
  roty(1,:) = [  cos(ry)   0     sin(ry) ];
  roty(3,:) = [ -sin(ry)   0     cos(ry) ];
end

if rz~=0
  % rotation around z-axis
  rotz(1,:) = [  cos(rz)  -sin(rz)   0 ];
  rotz(2,:) = [  sin(rz)   cos(rz)   0 ];
end

if nargin==1
  % compute and return the homogenous transformation matrix
  rotx(4,4) = 1;
  roty(4,4) = 1;
  rotz(4,4) = 1;
  output = rotz * roty * rotx;
else
  % apply the transformation on the input points
  output = ((rotz * roty * rotx) * input')';
end

